# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.interpolate.dfitpack, version: $Revision: $
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
__version__: bytes
def bispeu(tx, ty, c, kx, ky, x, y) -> typing.Any:
    "z,ier = bispeu(tx,ty,c,kx,ky,x,y)\n\nWrapper for ``bispeu``.\n\nParameters\n----------\ntx : input rank-1 array('d') with bounds (nx)\nty : input rank-1 array('d') with bounds (ny)\nc : input rank-1 array('d') with bounds ((nx-kx-1)*(ny-ky-1))\nkx : input int\nky : input int\nx : input rank-1 array('d') with bounds (m)\ny : input rank-1 array('d') with bounds (m)\n\nReturns\n-------\nz : rank-1 array('d') with bounds (m)\nier : int\n"
    ...

def bispev(tx, ty, c, kx, ky, x, y) -> typing.Any:
    "z,ier = bispev(tx,ty,c,kx,ky,x,y)\n\nWrapper for ``bispev``.\n\nParameters\n----------\ntx : input rank-1 array('d') with bounds (nx)\nty : input rank-1 array('d') with bounds (ny)\nc : input rank-1 array('d') with bounds ((nx-kx-1)*(ny-ky-1))\nkx : input int\nky : input int\nx : input rank-1 array('d') with bounds (mx)\ny : input rank-1 array('d') with bounds (my)\n\nReturns\n-------\nz : rank-2 array('d') with bounds (mx,my)\nier : int\n"
    ...

def curfit(iopt, x, y, w, t, wrk, iwrk, xb=..., xe=..., k=..., s=...) -> typing.Any:
    "n,c,fp,ier = curfit(iopt,x,y,w,t,wrk,iwrk,[xb,xe,k,s])\n\nWrapper for ``curfit``.\n\nParameters\n----------\niopt : input int\nx : input rank-1 array('d') with bounds (m)\ny : input rank-1 array('d') with bounds (m)\nw : input rank-1 array('d') with bounds (m)\nt : in/output rank-1 array('d') with bounds (nest)\nwrk : in/output rank-1 array('d') with bounds (lwrk)\niwrk : in/output rank-1 array('i') with bounds (nest)\n\nOther Parameters\n----------------\nxb : input float, optional\n    Default: x[0]\nxe : input float, optional\n    Default: x[m-1]\nk : input int, optional\n    Default: 3\ns : input float, optional\n    Default: 0.0\n\nReturns\n-------\nn : int\nc : rank-1 array('d') with bounds (n)\nfp : float\nier : int\n"
    ...

def dblint(tx, ty, c, kx, ky, xb, xe, yb, ye) -> typing.Any:
    "dblint = dblint(tx,ty,c,kx,ky,xb,xe,yb,ye)\n\nWrapper for ``dblint``.\n\nParameters\n----------\ntx : input rank-1 array('d') with bounds (nx)\nty : input rank-1 array('d') with bounds (ny)\nc : input rank-1 array('d') with bounds ((nx-kx-1)*(ny-ky-1))\nkx : input int\nky : input int\nxb : input float\nxe : input float\nyb : input float\nye : input float\n\nReturns\n-------\ndblint : float\n"
    ...

def fpchec(x, t, k) -> typing.Any:
    "ier = fpchec(x,t,k)\n\nWrapper for ``fpchec``.\n\nParameters\n----------\nx : input rank-1 array('d') with bounds (m)\nt : input rank-1 array('d') with bounds (n)\nk : input int\n\nReturns\n-------\nier : int\n"
    ...

def fpcurf0(x, y, k, w=..., xb=..., xe=..., s=..., nest=...) -> typing.Any:
    "x,y,w,xb,xe,k,s,n,t,c,fp,fpint,nrdata,ier = fpcurf0(x,y,k,[w,xb,xe,s,nest])\n\nWrapper for ``fpcurf0``.\n\nParameters\n----------\nx : input rank-1 array('d') with bounds (m)\ny : input rank-1 array('d') with bounds (m)\nk : input int\n\nOther Parameters\n----------------\nw : input rank-1 array('d') with bounds (m), optional\n    Default: 1.0\nxb : input float, optional\n    Default: x[0]\nxe : input float, optional\n    Default: x[m-1]\ns : input float, optional\n    Default: m\nnest : input int, optional\n    Default: (s==0.0?m+k+1:MAX(m/2,2*k1))\n\nReturns\n-------\nx : rank-1 array('d') with bounds (m)\ny : rank-1 array('d') with bounds (m)\nw : rank-1 array('d') with bounds (m)\nxb : float\nxe : float\nk : int\ns : float\nn : int\nt : rank-1 array('d') with bounds (nest)\nc : rank-1 array('d') with bounds (nest)\nfp : float\nfpint : rank-1 array('d') with bounds (nest)\nnrdata : rank-1 array('i') with bounds (nest)\nier : int\n"
    ...

def fpcurf1(x, y, w, xb, xe, k, s, n, t, c, fp, fpint, nrdata, ier, overwrite_x=..., overwrite_y=..., overwrite_w=..., overwrite_t=..., overwrite_c=..., overwrite_fpint=..., overwrite_nrdata=...) -> typing.Any:
    "x,y,w,xb,xe,k,s,n,t,c,fp,fpint,nrdata,ier = fpcurf1(x,y,w,xb,xe,k,s,n,t,c,fp,fpint,nrdata,ier,[overwrite_x,overwrite_y,overwrite_w,overwrite_t,overwrite_c,overwrite_fpint,overwrite_nrdata])\n\nWrapper for ``fpcurf1``.\n\nParameters\n----------\nx : input rank-1 array('d') with bounds (m)\ny : input rank-1 array('d') with bounds (m)\nw : input rank-1 array('d') with bounds (m)\nxb : input float\nxe : input float\nk : input int\ns : input float\nn : input int\nt : input rank-1 array('d') with bounds (nest)\nc : input rank-1 array('d') with bounds (nest)\nfp : input float\nfpint : input rank-1 array('d') with bounds (nest)\nnrdata : input rank-1 array('i') with bounds (nest)\nier : input int\n\nOther Parameters\n----------------\noverwrite_x : input int, optional\n    Default: 1\noverwrite_y : input int, optional\n    Default: 1\noverwrite_w : input int, optional\n    Default: 1\noverwrite_t : input int, optional\n    Default: 1\noverwrite_c : input int, optional\n    Default: 1\noverwrite_fpint : input int, optional\n    Default: 1\noverwrite_nrdata : input int, optional\n    Default: 1\n\nReturns\n-------\nx : rank-1 array('d') with bounds (m)\ny : rank-1 array('d') with bounds (m)\nw : rank-1 array('d') with bounds (m)\nxb : float\nxe : float\nk : int\ns : float\nn : int\nt : rank-1 array('d') with bounds (nest)\nc : rank-1 array('d') with bounds (nest)\nfp : float\nfpint : rank-1 array('d') with bounds (nest)\nnrdata : rank-1 array('i') with bounds (nest)\nier : int\n"
    ...

def fpcurfm1(x, y, k, t, w=..., xb=..., xe=..., overwrite_t=...) -> typing.Any:
    "x,y,w,xb,xe,k,s,n,t,c,fp,fpint,nrdata,ier = fpcurfm1(x,y,k,t,[w,xb,xe,overwrite_t])\n\nWrapper for ``fpcurfm1``.\n\nParameters\n----------\nx : input rank-1 array('d') with bounds (m)\ny : input rank-1 array('d') with bounds (m)\nk : input int\nt : input rank-1 array('d') with bounds (n)\n\nOther Parameters\n----------------\nw : input rank-1 array('d') with bounds (m), optional\n    Default: 1.0\nxb : input float, optional\n    Default: x[0]\nxe : input float, optional\n    Default: x[m-1]\noverwrite_t : input int, optional\n    Default: 1\n\nReturns\n-------\nx : rank-1 array('d') with bounds (m)\ny : rank-1 array('d') with bounds (m)\nw : rank-1 array('d') with bounds (m)\nxb : float\nxe : float\nk : int\ns : float\nn : int\nt : rank-1 array('d') with bounds (n)\nc : rank-1 array('d') with bounds (nest)\nfp : float\nfpint : rank-1 array('d') with bounds (nest)\nnrdata : rank-1 array('i') with bounds (nest)\nier : int\n"
    ...

def parcur(iopt, ipar, idim, u, x, w, ub, ue, t, wrk, iwrk, k=..., s=...) -> typing.Any:
    "n,c,fp,ier = parcur(iopt,ipar,idim,u,x,w,ub,ue,t,wrk,iwrk,[k,s])\n\nWrapper for ``parcur``.\n\nParameters\n----------\niopt : input int\nipar : input int\nidim : input int\nu : in/output rank-1 array('d') with bounds (m)\nx : input rank-1 array('d') with bounds (mx)\nw : input rank-1 array('d') with bounds (m)\nub : input float\nue : input float\nt : in/output rank-1 array('d') with bounds (nest)\nwrk : in/output rank-1 array('d') with bounds (lwrk)\niwrk : in/output rank-1 array('i') with bounds (nest)\n\nOther Parameters\n----------------\nk : input int, optional\n    Default: 3.0\ns : input float, optional\n    Default: 0.0\n\nReturns\n-------\nn : int\nc : rank-1 array('d') with bounds (nc)\nfp : float\nier : int\n"
    ...

def parder(tx, ty, c, kx, ky, nux, nuy, x, y) -> typing.Any:
    "z,ier = parder(tx,ty,c,kx,ky,nux,nuy,x,y)\n\nWrapper for ``parder``.\n\nParameters\n----------\ntx : input rank-1 array('d') with bounds (nx)\nty : input rank-1 array('d') with bounds (ny)\nc : input rank-1 array('d') with bounds ((nx-kx-1)*(ny-ky-1))\nkx : input int\nky : input int\nnux : input int\nnuy : input int\nx : input rank-1 array('d') with bounds (mx)\ny : input rank-1 array('d') with bounds (my)\n\nReturns\n-------\nz : rank-2 array('d') with bounds (mx,my)\nier : int\n"
    ...

def pardeu(tx, ty, c, kx, ky, nux, nuy, x, y) -> typing.Any:
    "z,ier = pardeu(tx,ty,c,kx,ky,nux,nuy,x,y)\n\nWrapper for ``pardeu``.\n\nParameters\n----------\ntx : input rank-1 array('d') with bounds (nx)\nty : input rank-1 array('d') with bounds (ny)\nc : input rank-1 array('d') with bounds ((nx-kx-1)*(ny-ky-1))\nkx : input int\nky : input int\nnux : input int\nnuy : input int\nx : input rank-1 array('d') with bounds (m)\ny : input rank-1 array('d') with bounds (m)\n\nReturns\n-------\nz : rank-1 array('d') with bounds (m)\nier : int\n"
    ...

def percur(iopt, x, y, w, t, wrk, iwrk, k=..., s=...) -> typing.Any:
    "n,c,fp,ier = percur(iopt,x,y,w,t,wrk,iwrk,[k,s])\n\nWrapper for ``percur``.\n\nParameters\n----------\niopt : input int\nx : input rank-1 array('d') with bounds (m)\ny : input rank-1 array('d') with bounds (m)\nw : input rank-1 array('d') with bounds (m)\nt : in/output rank-1 array('d') with bounds (nest)\nwrk : in/output rank-1 array('d') with bounds (lwrk)\niwrk : in/output rank-1 array('i') with bounds (nest)\n\nOther Parameters\n----------------\nk : input int, optional\n    Default: 3\ns : input float, optional\n    Default: 0.0\n\nReturns\n-------\nn : int\nc : rank-1 array('d') with bounds (n)\nfp : float\nier : int\n"
    ...

def regrid_smth(x, y, z, xb=..., xe=..., yb=..., ye=..., kx=..., ky=..., s=...) -> typing.Any:
    "nx,tx,ny,ty,c,fp,ier = regrid_smth(x,y,z,[xb,xe,yb,ye,kx,ky,s])\n\nWrapper for ``regrid_smth``.\n\nParameters\n----------\nx : input rank-1 array('d') with bounds (mx)\ny : input rank-1 array('d') with bounds (my)\nz : input rank-1 array('d') with bounds (mx*my)\n\nOther Parameters\n----------------\nxb : input float, optional\n    Default: dmin(x,mx)\nxe : input float, optional\n    Default: dmax(x,mx)\nyb : input float, optional\n    Default: dmin(y,my)\nye : input float, optional\n    Default: dmax(y,my)\nkx : input int, optional\n    Default: 3\nky : input int, optional\n    Default: 3\ns : input float, optional\n    Default: 0.0\n\nReturns\n-------\nnx : int\ntx : rank-1 array('d') with bounds (nxest)\nny : int\nty : rank-1 array('d') with bounds (nyest)\nc : rank-1 array('d') with bounds ((nxest-kx-1)*(nyest-ky-1))\nfp : float\nier : int\n"
    ...

def regrid_smth_spher(iopt, ider, u, v, r, r0=..., r1=..., s=...) -> typing.Any:
    "nu,tu,nv,tv,c,fp,ier = regrid_smth_spher(iopt,ider,u,v,r,[r0,r1,s])\n\nWrapper for ``regrid_smth_spher``.\n\nParameters\n----------\niopt : input rank-1 array('i') with bounds (3)\nider : input rank-1 array('i') with bounds (4)\nu : input rank-1 array('d') with bounds (mu)\nv : input rank-1 array('d') with bounds (mv)\nr : input rank-1 array('d') with bounds (mu*mv)\n\nOther Parameters\n----------------\nr0 : input float\nr1 : input float\ns : input float, optional\n    Default: 0.0\n\nReturns\n-------\nnu : int\ntu : rank-1 array('d') with bounds (nuest)\nnv : int\ntv : rank-1 array('d') with bounds (nvest)\nc : rank-1 array('d') with bounds ((nuest-4)*(nvest-4))\nfp : float\nier : int\n"
    ...

def spalde(t, c, k, x) -> typing.Any:
    "d,ier = spalde(t,c,k,x)\n\nWrapper for ``spalde``.\n\nParameters\n----------\nt : input rank-1 array('d') with bounds (n)\nc : input rank-1 array('d') with bounds (n)\nk : input int\nx : input float\n\nReturns\n-------\nd : rank-1 array('d') with bounds (k + 1)\nier : int\n"
    ...

def spherfit_lsq(teta, phi, r, tt, tp, w=..., eps=..., overwrite_tt=..., overwrite_tp=...) -> typing.Any:
    "tt,tp,c,fp,ier = spherfit_lsq(teta,phi,r,tt,tp,[w,eps,overwrite_tt,overwrite_tp])\n\nWrapper for ``spherfit_lsq``.\n\nParameters\n----------\nteta : input rank-1 array('d') with bounds (m)\nphi : input rank-1 array('d') with bounds (m)\nr : input rank-1 array('d') with bounds (m)\ntt : input rank-1 array('d') with bounds (ntest)\ntp : input rank-1 array('d') with bounds (npest)\n\nOther Parameters\n----------------\nw : input rank-1 array('d') with bounds (m), optional\n    Default: 1.0\neps : input float, optional\n    Default: 1e-16\noverwrite_tt : input int, optional\n    Default: 1\noverwrite_tp : input int, optional\n    Default: 1\n\nReturns\n-------\ntt : rank-1 array('d') with bounds (ntest)\ntp : rank-1 array('d') with bounds (npest)\nc : rank-1 array('d') with bounds ((nt-4)*(np-4))\nfp : float\nier : int\n"
    ...

def spherfit_smth(teta, phi, r, w=..., s=..., eps=...) -> typing.Any:
    "nt,tt,np,tp,c,fp,ier = spherfit_smth(teta,phi,r,[w,s,eps])\n\nWrapper for ``spherfit_smth``.\n\nParameters\n----------\nteta : input rank-1 array('d') with bounds (m)\nphi : input rank-1 array('d') with bounds (m)\nr : input rank-1 array('d') with bounds (m)\n\nOther Parameters\n----------------\nw : input rank-1 array('d') with bounds (m), optional\n    Default: 1.0\ns : input float, optional\n    Default: m\neps : input float, optional\n    Default: 1e-16\n\nReturns\n-------\nnt : int\ntt : rank-1 array('d') with bounds (ntest)\nnp : int\ntp : rank-1 array('d') with bounds (npest)\nc : rank-1 array('d') with bounds ((ntest-4)*(npest-4))\nfp : float\nier : int\n"
    ...

def splder(t, c, k, x, nu=..., e=...) -> typing.Any:
    "y = splder(t,c,k,x,[nu,e])\n\nWrapper for ``splder``.\n\nParameters\n----------\nt : input rank-1 array('d') with bounds (n)\nc : input rank-1 array('d') with bounds (n)\nk : input int\nx : input rank-1 array('d') with bounds (m)\n\nOther Parameters\n----------------\nnu : input int, optional\n    Default: 1\ne : input int, optional\n    Default: 0\n\nReturns\n-------\ny : rank-1 array('d') with bounds (m)\n"
    ...

def splev(t, c, k, x, e=...) -> typing.Any:
    "y = splev(t,c,k,x,[e])\n\nWrapper for ``splev``.\n\nParameters\n----------\nt : input rank-1 array('d') with bounds (n)\nc : input rank-1 array('d') with bounds (n)\nk : input int\nx : input rank-1 array('d') with bounds (m)\n\nOther Parameters\n----------------\ne : input int, optional\n    Default: 0\n\nReturns\n-------\ny : rank-1 array('d') with bounds (m)\n"
    ...

def splint(t, c, k, a, b) -> typing.Any:
    "splint = splint(t,c,k,a,b)\n\nWrapper for ``splint``.\n\nParameters\n----------\nt : input rank-1 array('d') with bounds (n)\nc : input rank-1 array('d') with bounds (n)\nk : input int\na : input float\nb : input float\n\nReturns\n-------\nsplint : float\n"
    ...

def sproot(t, c, mest=...) -> typing.Any:
    "zero,m,ier = sproot(t,c,[mest])\n\nWrapper for ``sproot``.\n\nParameters\n----------\nt : input rank-1 array('d') with bounds (n)\nc : input rank-1 array('d') with bounds (n)\n\nOther Parameters\n----------------\nmest : input int, optional\n    Default: 3*(n-7)\n\nReturns\n-------\nzero : rank-1 array('d') with bounds (mest)\nm : int\nier : int\n"
    ...

def surfit_lsq(x, y, z, nx, tx, ny, ty, w=..., xb=..., xe=..., yb=..., ye=..., kx=..., ky=..., eps=..., lwrk2=..., overwrite_tx=..., overwrite_ty=...) -> typing.Any:
    "tx,ty,c,fp,ier = surfit_lsq(x,y,z,nx,tx,ny,ty,[w,xb,xe,yb,ye,kx,ky,eps,lwrk2,overwrite_tx,overwrite_ty])\n\nWrapper for ``surfit_lsq``.\n\nParameters\n----------\nx : input rank-1 array('d') with bounds (m)\ny : input rank-1 array('d') with bounds (m)\nz : input rank-1 array('d') with bounds (m)\nnx : input int\ntx : input rank-1 array('d') with bounds (nmax)\nny : input int\nty : input rank-1 array('d') with bounds (nmax)\n\nOther Parameters\n----------------\nw : input rank-1 array('d') with bounds (m), optional\n    Default: 1.0\nxb : input float, optional\n    Default: calc_b(x,m,tx,nx)\nxe : input float, optional\n    Default: calc_e(x,m,tx,nx)\nyb : input float, optional\n    Default: calc_b(y,m,ty,ny)\nye : input float, optional\n    Default: calc_e(y,m,ty,ny)\nkx : input int, optional\n    Default: 3\nky : input int, optional\n    Default: 3\neps : input float, optional\n    Default: 1e-16\noverwrite_tx : input int, optional\n    Default: 1\noverwrite_ty : input int, optional\n    Default: 1\nlwrk2 : input int, optional\n    Default: calc_surfit_lwrk2(m,kx,ky,nxest,nyest)\n\nReturns\n-------\ntx : rank-1 array('d') with bounds (nmax)\nty : rank-1 array('d') with bounds (nmax)\nc : rank-1 array('d') with bounds ((nx-kx-1)*(ny-ky-1))\nfp : float\nier : int\n"
    ...

def surfit_smth(x, y, z, w=..., xb=..., xe=..., yb=..., ye=..., kx=..., ky=..., s=..., nxest=..., nyest=..., eps=..., lwrk2=...) -> typing.Any:
    "nx,tx,ny,ty,c,fp,wrk1,ier = surfit_smth(x,y,z,[w,xb,xe,yb,ye,kx,ky,s,nxest,nyest,eps,lwrk2])\n\nWrapper for ``surfit_smth``.\n\nParameters\n----------\nx : input rank-1 array('d') with bounds (m)\ny : input rank-1 array('d') with bounds (m)\nz : input rank-1 array('d') with bounds (m)\n\nOther Parameters\n----------------\nw : input rank-1 array('d') with bounds (m), optional\n    Default: 1.0\nxb : input float, optional\n    Default: dmin(x,m)\nxe : input float, optional\n    Default: dmax(x,m)\nyb : input float, optional\n    Default: dmin(y,m)\nye : input float, optional\n    Default: dmax(y,m)\nkx : input int, optional\n    Default: 3\nky : input int, optional\n    Default: 3\ns : input float, optional\n    Default: m\nnxest : input int, optional\n    Default: imax(kx+1+sqrt(m/2),2*(kx+1))\nnyest : input int, optional\n    Default: imax(ky+1+sqrt(m/2),2*(ky+1))\neps : input float, optional\n    Default: 1e-16\nlwrk2 : input int, optional\n    Default: calc_surfit_lwrk2(m,kx,ky,nxest,nyest)\n\nReturns\n-------\nnx : int\ntx : rank-1 array('d') with bounds (nmax)\nny : int\nty : rank-1 array('d') with bounds (nmax)\nc : rank-1 array('d') with bounds ((nxest-kx-1)*(nyest-ky-1))\nfp : float\nwrk1 : rank-1 array('d') with bounds (lwrk1)\nier : int\n"
    ...

def types() -> typing.Any:
    "'i'-scalar\n"
    ...

def __getattr__(name) -> typing.Any:
    ...




from typing import Any, ClassVar, NamedTuple
from collections import Counter
from contextlib import suppress as suppress
from numpy import str_, ndarray
from . import is_scalar_nan as is_scalar_nan

import numpy as np


class MissingValues(NamedTuple):

    nan: ClassVar[bool] = ...
    none: ClassVar[bool] = ...

    def to_list(self) -> list[float | Any]:
        ...


class _nandict(dict):
    def __init__(
        self, mapping: dict[str, int] | dict[str | float, int] | dict[str_, int]
    ) -> None:
        ...

    def __missing__(self, key):
        ...


class _NaNCounter(Counter):
    def __init__(self, items: ndarray) -> None:
        ...

    def __missing__(self, key):
        ...






# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division
import weakref
import numpy as np

from .. import gloo
from ..util.event import EmitterGroup, Event
from ..util import logger, Frozen
from .shaders import StatementList, MultiProgram
from .transforms import TransformSystem
from typing import Any, Sequence
from vispy.gloo.program import Program

class VisualShare(object):
    def __init__(self): ...

class BaseVisual(Frozen):
    def __init__(self, vshare: VisualShare | None = None): ...
    @property
    def transform(self): ...
    @transform.setter
    def transform(self, tr): ...
    @property
    def transforms(self): ...
    @transforms.setter
    def transforms(self, trs): ...
    def get_transform(self, map_from: str = "visual", map_to: str = "render"): ...
    @property
    def visible(self): ...
    @visible.setter
    def visible(self, v): ...
    def view(self): ...
    def draw(self): ...
    def attach(self, filt: Any, view: None | VisualView = None): ...
    def detach(self, filt: Any, view: None | VisualView = None): ...
    def bounds(self, axis: int, view: VisualView | None = None): ...
    def _compute_bounds(self, axis, view): ...
    def _bounds_changed(self): ...
    def update(self): ...
    def _transform_changed(self, event=None): ...

class BaseVisualView(object):
    def __init__(self, visual): ...
    @property
    def visual(self): ...
    def _prepare_draw(self, view=None): ...
    def _prepare_transforms(self, view): ...
    def _compute_bounds(self, axis, view): ...
    def __repr__(self): ...

class Visual(BaseVisual):
    def __init__(
        self,
        vcode: str = "",
        fcode: str = "",
        gcode: str | None = None,
        program: None | Program = None,
        vshare: VisualShare | None = None,
    ): ...
    def set_gl_state(self, preset: str | None = None, **kwargs): ...
    def update_gl_state(self, *args, **kwargs): ...
    def _compute_bounds(self, axis, view): ...
    def _prepare_draw(self, view=None): ...
    def _prepare_transforms(self, view): ...
    @property
    def shared_program(self): ...
    @property
    def view_program(self): ...
    @property
    def _draw_mode(self): ...
    @_draw_mode.setter
    def _draw_mode(self, m): ...
    @property
    def _index_buffer(self): ...
    @_index_buffer.setter
    def _index_buffer(self, buf): ...
    def draw(self): ...
    def _configure_gl_state(self): ...
    def _get_hook(self, shader, name): ...
    def attach(self, filt: Any, view: None | VisualView = None): ...
    def detach(self, filt: Any, view: None | VisualView = None): ...

class VisualView(BaseVisualView, Visual):
    def __init__(self, visual): ...

class CompoundVisual(BaseVisual):
    def __init__(self, subvisuals: Sequence[BaseVisual]): ...
    def add_subvisual(self, visual: Visual): ...
    def remove_subvisual(self, visual: Visual): ...
    def _subv_update(self, event): ...
    def _transform_changed(self, event=None): ...
    def draw(self): ...
    def _prepare_draw(self, view): ...
    def _prepare_transforms(self, view): ...
    def set_gl_state(self, preset: str | None = None, **kwargs): ...
    def update_gl_state(self, *args, **kwargs): ...
    def attach(self, filt: Any, view: None | VisualView = None): ...
    def detach(self, filt: Any, view: None | VisualView = None): ...
    def _compute_bounds(self, axis, view): ...

class CompoundVisualView(BaseVisualView, CompoundVisual):
    def __init__(self, visual): ...

class updating_property:
    def __init__(self, fget=None, fset=None, doc=None): ...
    def __get__(self, obj, objtype=None): ...
    def __set__(self, obj, value): ...
    def __delete__(self, obj): ...
    def setter(self, fset): ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.stats._stats, version: unspecified
import typing
import builtins as _mod_builtins

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
__pyx_capi__: dict
def __pyx_unpickle_Enum() -> typing.Any:
    ...

__test__: dict
def _center_distance_matrix() -> typing.Any:
    ...

def _kendall_dis() -> typing.Any:
    ...

def _local_correlations() -> typing.Any:
    ...

def _local_covariance() -> typing.Any:
    ...

def _rank_distance_matrix() -> typing.Any:
    ...

def _toint64() -> typing.Any:
    ...

def _transform_distance_matrix() -> typing.Any:
    ...

def _weightedrankedtau(x, y, rank, weigher, additive) -> typing.Any:
    ...

def gaussian_kernel_estimate() -> typing.Any:
    '\n    def gaussian_kernel_estimate(points, real[:, :] values, xi, precision)\n\n    Evaluate a multivariate Gaussian kernel estimate.\n\n    Parameters\n    ----------\n    points : array_like with shape (n, d)\n        Data points to estimate from in d dimenions.\n    values : real[:, :] with shape (n, p)\n        Multivariate values associated with the data points.\n    xi : array_like with shape (m, d)\n        Coordinates to evaluate the estimate at in d dimensions.\n    precision : array_like with shape (d, d)\n        Precision matrix for the Gaussian kernel.\n\n    Returns\n    -------\n    estimate : double[:, :] with shape (m, p)\n        Multivariate Gaussian kernel estimate evaluated at the input coordinates.\n    '
    ...

def geninvgauss_logpdf() -> typing.Any:
    ...

def von_mises_cdf() -> typing.Any:
    ...

def __getattr__(name) -> typing.Any:
    ...




from numpy.typing import ArrayLike
import math

import numpy as np
from numpy import arctan2, exp, pi, sqrt

from .. import draw

from .._shared.filters import gaussian
from .._shared.utils import check_nD

def daisy(
    image,
    step: int = 4,
    radius: int = 15,
    rings: int = 3,
    histograms: int = 8,
    orientations: int = 8,
    normalization="l1",
    sigmas=None,
    ring_radii=None,
    visualize: bool = False,
): ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.integrate.vode, version: $Revision: $
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
__version__: bytes
def dvode(f, jac, y, t, tout, rtol, atol, itask, istate, rwork, iwork, mf, f_extra_args=..., jac_extra_args=..., overwrite_y=...) -> typing.Any:
    "y,t,istate = dvode(f,jac,y,t,tout,rtol,atol,itask,istate,rwork,iwork,mf,[f_extra_args,jac_extra_args,overwrite_y])\n\nWrapper for ``dvode``.\n\nParameters\n----------\nf : call-back function\njac : call-back function\ny : input rank-1 array('d') with bounds (neq)\nt : input float\ntout : input float\nrtol : input rank-1 array('d') with bounds (*)\natol : input rank-1 array('d') with bounds (*)\nitask : input int\nistate : input int\nrwork : input rank-1 array('d') with bounds (lrw)\niwork : input rank-1 array('i') with bounds (liw)\nmf : input int\n\nOther Parameters\n----------------\nf_extra_args : input tuple, optional\n    Default: ()\njac_extra_args : input tuple, optional\n    Default: ()\noverwrite_y : input int, optional\n    Default: 0\n\nReturns\n-------\ny : rank-1 array('d') with bounds (neq)\nt : float\nistate : int\n\nNotes\n-----\nCall-back functions::\n\n  def f(t,y): return ydot\n  Required arguments:\n    t : input float\n    y : input rank-1 array('d') with bounds (n)\n  Return objects:\n    ydot : rank-1 array('d') with bounds (n)\n  def jac(t,y): return jac\n  Required arguments:\n    t : input float\n    y : input rank-1 array('d') with bounds (n)\n  Return objects:\n    jac : rank-2 array('d') with bounds (nrowpd,n)\n"
    ...

def types() -> typing.Any:
    "'i'-scalar\n"
    ...

def zvode(f, jac, y, t, tout, rtol, atol, itask, istate, zwork, rwork, iwork, mf, f_extra_args=..., jac_extra_args=..., overwrite_y=...) -> typing.Any:
    "y,t,istate = zvode(f,jac,y,t,tout,rtol,atol,itask,istate,zwork,rwork,iwork,mf,[f_extra_args,jac_extra_args,overwrite_y])\n\nWrapper for ``zvode``.\n\nParameters\n----------\nf : call-back function\njac : call-back function\ny : input rank-1 array('D') with bounds (neq)\nt : input float\ntout : input float\nrtol : input rank-1 array('d') with bounds (*)\natol : input rank-1 array('d') with bounds (*)\nitask : input int\nistate : input int\nzwork : input rank-1 array('D') with bounds (lzw)\nrwork : input rank-1 array('d') with bounds (lrw)\niwork : input rank-1 array('i') with bounds (liw)\nmf : input int\n\nOther Parameters\n----------------\nf_extra_args : input tuple, optional\n    Default: ()\njac_extra_args : input tuple, optional\n    Default: ()\noverwrite_y : input int, optional\n    Default: 0\n\nReturns\n-------\ny : rank-1 array('D') with bounds (neq)\nt : float\nistate : int\n\nNotes\n-----\nCall-back functions::\n\n  def f(t,y): return ydot\n  Required arguments:\n    t : input float\n    y : input rank-1 array('D') with bounds (n)\n  Return objects:\n    ydot : rank-1 array('D') with bounds (n)\n  def jac(t,y): return jac\n  Required arguments:\n    t : input float\n    y : input rank-1 array('D') with bounds (n)\n  Return objects:\n    jac : rank-2 array('D') with bounds (nrowpd,n)\n"
    ...

def __getattr__(name) -> typing.Any:
    ...




from typing import Any, ClassVar, Literal, TypeVar
from numpy.random import RandomState
from scipy import stats as stats
from ..base import BaseEstimator
from ..exceptions import ConvergenceWarning as ConvergenceWarning
from ..preprocessing import normalize as normalize
from ..linear_model import BayesianRidge as BayesianRidge
from ..utils.validation import (
    FLOAT_DTYPES as FLOAT_DTYPES,
    check_is_fitted as check_is_fitted,
)
from ._base import MissingIndicator
from collections import namedtuple as namedtuple
from numpy import ndarray
from ..utils._param_validation import (
    HasMethods as HasMethods,
    Interval as Interval,
    StrOptions as StrOptions,
)
from numbers import Integral as Integral, Real as Real
from ..base import clone as clone
from time import time as time
from ..utils import (
    check_array as check_array,
    check_random_state as check_random_state,
    is_scalar_nan as is_scalar_nan,
)
from ._base import _BaseImputer, SimpleImputer
from .._typing import Int, Float, ArrayLike, MatrixLike

IterativeImputer_Self = TypeVar("IterativeImputer_Self", bound="IterativeImputer")

import warnings
import numpy as np


_ImputerTriplet = ...


class IterativeImputer(_BaseImputer):
    random_state_: RandomState = ...
    indicator_: MissingIndicator = ...
    n_features_with_missing_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_iter_: int = ...
    imputation_sequence_: list[tuple] = ...
    initial_imputer_: SimpleImputer = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        estimator: None | BaseEstimator = None,
        *,
        missing_values: float | int = ...,
        sample_posterior: bool = False,
        max_iter: Int = 10,
        tol: Float = 1e-3,
        n_nearest_features: None | Int = None,
        initial_strategy: Literal[
            "mean", "median", "most_frequent", "constant", "mean"
        ] = "mean",
        imputation_order: Literal[
            "ascending", "descending", "roman", "arabic", "random", "ascending"
        ] = "ascending",
        skip_complete: bool = False,
        min_value: float | ArrayLike = ...,
        max_value: float | ArrayLike = ...,
        verbose: Int = 0,
        random_state: RandomState | None | Int = None,
        add_indicator: bool = False,
        keep_empty_features: bool = False,
    ) -> None:
        ...

    def fit_transform(self, X: MatrixLike, y: Any = None) -> ndarray:
        ...

    def transform(self, X: MatrixLike) -> ndarray:
        ...

    def fit(
        self: IterativeImputer_Self, X: MatrixLike, y: Any = None
    ) -> IterativeImputer_Self:
        ...

    def get_feature_names_out(self, input_features: None | ArrayLike = None) -> ndarray:
        ...



from networkx import DiGraph

# See https://github.com/networkx/networkx/pull/1474
# Copyright 2011 Reya Group <http://www.reyagroup.com>
# Copyright 2011 Alex Levenson <alex@isnotinvain.com>
# Copyright 2011 Diederik van Liere <diederik.vanliere@rotman.utoronto.ca>
from ..classes import DiGraph

__all__ = ["triad_graph"]

#: Dictionary mapping triad name to list of directed edges in the
#: digraph representation of that triad (with nodes 'a', 'b', and 'c').
TRIAD_EDGES: dict = ...

def triad_graph(triad_name: str) -> DiGraph: ...



from typing import Sequence
from cycler import Cycler
import ast

interactive_bk = ...
non_interactive_bk = ...
all_backends = ...

class ValidateInStrings:
    def __init__(
        self,
        key: str,
        valid: Sequence[str] | _ignorecase,
        ignorecase: bool = ...,
        *,
        _deprecated_since=...
    ) -> None: ...
    def __call__(self, s: str) -> str: ...

def validate_any(s): ...

validate_anylist = ...

def validate_bool(b) -> bool: ...
def validate_axisbelow(s) -> bool | str: ...
def validate_dpi(s) -> float | str: ...

validate_string = ...
validate_string_or_None = ...
validate_stringlist = ...
validate_int = ...
validate_int_or_None = ...
validate_float = ...
validate_float_or_None = ...
validate_floatlist = ...

def validate_fonttype(s) -> int: ...
def validate_backend(s) -> str: ...
def validate_color_or_inherit(s) -> str: ...
def validate_color_or_auto(s) -> str: ...
def validate_color_for_prop_cycle(s) -> str: ...
def validate_color(s) -> str: ...

validate_colorlist = ...

def validate_aspect(s) -> str: ...
def validate_fontsize_None(s) -> None: ...
def validate_fontsize(s) -> float | str: ...

validate_fontsizelist = ...

def validate_fontweight(s) -> str: ...
def validate_fontstretch(s) -> str: ...
def validate_font_properties(s) -> str: ...
def validate_whiskers(s) -> float: ...
def validate_ps_distiller(s) -> None: ...

validate_fillstyle = ...
validate_fillstylelist = ...

def validate_markevery(s): ...

validate_markeverylist = ...

def validate_bbox(s) -> None: ...
def validate_sketch(s) -> None | tuple[float, ...]: ...
def validate_hatch(s): ...

validate_hatchlist = ...
validate_dashlist = ...

def cycler(*args, **kwargs) -> Cycler: ...

class _DunderChecker(ast.NodeVisitor):
    def visit_Attribute(self, node): ...

def validate_cycler(s) -> Cycler: ...
def validate_hist_bins(s) -> int: ...

class _ignorecase(list): ...



from numpy.typing import NDArray

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

import time
import os
import sys
import inspect
import base64
from subprocess import check_call, CalledProcessError
import numpy as np
from typing import Literal

from http.client import HTTPConnection
from urllib.parse import urlencode

from .. import scene, config
from ..io import read_png, write_png
from ..gloo.util import _screenshot
from ..util import run_subprocess
from . import IS_CI

tester: None = ...

def _get_tester(): ...
def assert_image_approved(image: NDArray | Literal["screenshot"], standard_file: str, message: str | None = None, **kwargs): ...
def assert_image_match(
    im1: NDArray,
    im2: NDArray,
    min_corr: float | None = 0.9,
    px_threshold: float = 50.0,
    px_count: int | None = None,
    max_px_diff: float | None = None,
    avg_px_diff: float | None = None,
    img_diff: float | None = None,
): ...
def _save_failed_test(data, expect, filename): ...
def make_diff_image(im1, im2): ...
def downsample(data, n, axis=0): ...

class ImageTester(scene.SceneCanvas):
    def __init__(self): ...
    def test(self, im1, im2, message): ...
    def on_key_press(self, event): ...

def get_test_data_repo(): ...
def git_cmd_base(path): ...
def git_status(path): ...
def git_commit_id(path, ref): ...



from typing import Any, Callable, ClassVar, Literal, TypeVar
from scipy import sparse as sparse
from numpy import ndarray
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from numbers import Integral as Integral, Real as Real
from ._dbscan_inner import dbscan_inner as dbscan_inner
from ..neighbors import NearestNeighbors as NearestNeighbors
from ..base import BaseEstimator, ClusterMixin
from .._typing import MatrixLike, Float, Int, ArrayLike

DBSCAN_Self = TypeVar("DBSCAN_Self", bound="DBSCAN")


# Author: Robert Layton <robertlayton@gmail.com>
#         Joel Nothman <joel.nothman@gmail.com>
#         Lars Buitinck
#
# License: BSD 3 clause

import warnings

import numpy as np


def dbscan(
    X: MatrixLike,
    eps: Float = 0.5,
    *,
    min_samples: Int = 5,
    metric: str | Callable = "minkowski",
    metric_params: None | dict = None,
    algorithm: Literal["auto", "ball_tree", "kd_tree", "brute", "auto"] = "auto",
    leaf_size: Int = 30,
    p: Float = 2,
    sample_weight: None | ArrayLike = None,
    n_jobs: None | Int = None,
) -> tuple[ndarray, ndarray]:
    ...


class DBSCAN(ClusterMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    labels_: ndarray = ...
    components_: ndarray = ...
    core_sample_indices_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        eps: Float = 0.5,
        *,
        min_samples: Int = 5,
        metric: str | Callable = "euclidean",
        metric_params: None | dict = None,
        algorithm: Literal["auto", "ball_tree", "kd_tree", "brute", "auto"] = "auto",
        leaf_size: Int = 30,
        p: None | Float = None,
        n_jobs: None | Int = None,
    ) -> None:
        ...

    def fit(
        self: DBSCAN_Self,
        X: MatrixLike,
        y: Any = None,
        sample_weight: None | ArrayLike = None,
    ) -> DBSCAN_Self:
        ...

    def fit_predict(
        self, X: MatrixLike, y: Any = None, sample_weight: None | ArrayLike = None
    ) -> ndarray:
        ...



from ...utils import not_implemented_for, py_random_state
from ...classes.graph import Graph

__all__ = ["average_clustering"]

@py_random_state(2)
def average_clustering(G: Graph, trials=1000, seed=None) -> float: ...




import numpy as np


class Criterion:

    y: np.ndarray
    sample_weight: np.ndarray

    sample_indices: np.ndarray
    start: int
    pos: int
    end: int

    n_outputs: int
    n_samples: int
    n_node_samples: int
    weighted_n_samples: float
    weighted_n_node_samples: float
    weighted_n_left: float
    weighted_n_right: float

    def init(
        self,
        y: np.ndarray,
        sample_weight: np.ndarray,
        weighted_n_samples: float,
        sample_indices: np.ndarray,
        start: int,
        end: int
    ) -> int: ...
    def reset(self) -> int: ...
    def reverse_reset(self) -> int: ...
    def update(self, new_pos: int) -> int: ...
    def node_impurity(self) -> float: ...
    def children_impurity(
        self,
        impurity_left: list[float],
        impurity_right: list[float]
    ) -> None: ...
    def node_value(
        self,
        dest: list[float]
    ) -> None: ...
    def impurity_improvement(
        self,
        impurity_parent: float,
        impurity_left: float,
        impurity_right: float
    ) -> float: ...
    def proxy_impurity_improvement(self) -> float: ...


class ClassificationCriterion(Criterion):


    n_classes: np.ndarray
    max_n_classes: int

    sum_total: np.ndarray
    sum_left: np.ndarray
    sum_right: np.ndarray

    
class RegressionCriterion(Criterion):

    sq_sum_total: float

    sum_total: np.ndarray
    sum_left: np.ndarray
    sum_right: np.ndarray



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014, Nicolas P. Rougier
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------
from .raw_segment_collection import RawSegmentCollection
from .agg_segment_collection import AggSegmentCollection

def SegmentCollection(mode="agg-fast", *args, **kwargs): ...



from typing import ClassVar, TypeVar
from scipy import linalg as linalg
from ._base import LinearModel
from numpy import ndarray
from ..utils.extmath import fast_logdet as fast_logdet
from scipy.linalg import pinvh as pinvh
from numbers import Integral as Integral, Real as Real
from ..utils._param_validation import Interval as Interval
from math import log as log
from ..base import RegressorMixin
from .._typing import Int, Float, ArrayLike, MatrixLike

BayesianRidge_Self = TypeVar("BayesianRidge_Self", bound="BayesianRidge")
ARDRegression_Self = TypeVar("ARDRegression_Self", bound="ARDRegression")

import numpy as np

###############################################################################
# BayesianRidge regression


class BayesianRidge(RegressorMixin, LinearModel):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    X_scale_: ndarray = ...
    X_offset_: ndarray = ...
    n_iter_: int = ...
    scores_: ArrayLike = ...
    sigma_: ArrayLike = ...
    lambda_: float = ...
    alpha_: float = ...
    intercept_: float = ...
    coef_: ArrayLike = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        n_iter: Int = 300,
        tol: Float = 1.0e-3,
        alpha_1: Float = 1.0e-6,
        alpha_2: Float = 1.0e-6,
        lambda_1: Float = 1.0e-6,
        lambda_2: Float = 1.0e-6,
        alpha_init: None | Float = None,
        lambda_init: None | Float = None,
        compute_score: bool = False,
        fit_intercept: bool = True,
        copy_X: bool = True,
        verbose: bool = False,
    ) -> None:
        ...

    def fit(
        self: BayesianRidge_Self,
        X: ArrayLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> BayesianRidge_Self:
        ...

    def predict(
        self, X: MatrixLike | ArrayLike, return_std: bool = False
    ) -> ArrayLike | tuple[ndarray, ndarray] | tuple[ArrayLike, ArrayLike]:
        ...


###############################################################################
# ARD (Automatic Relevance Determination) regression


class ARDRegression(RegressorMixin, LinearModel):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    X_scale_: float = ...
    X_offset_: float = ...
    intercept_: float = ...
    scores_: float = ...
    sigma_: ArrayLike = ...
    lambda_: ArrayLike = ...
    alpha_: float = ...
    coef_: ArrayLike = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        n_iter: Int = 300,
        tol: Float = 1.0e-3,
        alpha_1: Float = 1.0e-6,
        alpha_2: Float = 1.0e-6,
        lambda_1: Float = 1.0e-6,
        lambda_2: Float = 1.0e-6,
        compute_score: bool = False,
        threshold_lambda: Float = 1.0e4,
        fit_intercept: bool = True,
        copy_X: bool = True,
        verbose: bool = False,
    ) -> None:
        ...

    def fit(
        self: ARDRegression_Self, X: MatrixLike, y: ArrayLike
    ) -> ARDRegression_Self:
        ...

    def predict(
        self, X: MatrixLike | ArrayLike, return_std: bool = False
    ) -> tuple[ndarray, ndarray] | tuple[ArrayLike, ArrayLike]:
        ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.cluster._optimal_leaf_ordering, version: unspecified
import typing
import builtins as _mod_builtins

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
def __pyx_unpickle_Enum() -> typing.Any:
    ...

__test__: dict
def is_valid_dm(D, tol, throw, name, warning) -> typing.Any:
    '\n    Return True if input array is a valid distance matrix.\n\n    Distance matrices must be 2-dimensional numpy arrays.\n    They must have a zero-diagonal, and they must be symmetric.\n\n    Parameters\n    ----------\n    D : ndarray\n        The candidate object to test for validity.\n    tol : float, optional\n        The distance matrix should be symmetric. `tol` is the maximum\n        difference between entries ``ij`` and ``ji`` for the distance\n        metric to be considered symmetric.\n    throw : bool, optional\n        An exception is thrown if the distance matrix passed is not valid.\n    name : str, optional\n        The name of the variable to checked. This is useful if\n        throw is set to True so the offending variable can be identified\n        in the exception message when an exception is thrown.\n    warning : bool, optional\n        Instead of throwing an exception, a warning message is\n        raised.\n\n    Returns\n    -------\n    valid : bool\n        True if the variable `D` passed is a valid distance matrix.\n\n    Notes\n    -----\n    Small numerical differences in `D` and `D.T` and non-zeroness of\n    the diagonal are ignored if they are within the tolerance specified\n    by `tol`.\n\n    '
    ...

def is_valid_y(y, warning, throw, name) -> typing.Any:
    '\n    Return True if the input array is a valid condensed distance matrix.\n\n    Condensed distance matrices must be 1-dimensional numpy arrays.\n    Their length must be a binomial coefficient :math:`{n \\choose 2}`\n    for some positive integer n.\n\n    Parameters\n    ----------\n    y : ndarray\n        The condensed distance matrix.\n    warning : bool, optional\n        Invokes a warning if the variable passed is not a valid\n        condensed distance matrix. The warning message explains why\n        the distance matrix is not valid.  `name` is used when\n        referencing the offending variable.\n    throw : bool, optional\n        Throws an exception if the variable passed is not a valid\n        condensed distance matrix.\n    name : bool, optional\n        Used when referencing the offending variable in the\n        warning or exception message.\n\n    '
    ...

def optimal_leaf_ordering() -> typing.Any:
    '\n    Compute the optimal leaf order for Z (according to D) and return an \n    optimally sorted Z. \n\n    We start by sorting and relabelling Z and D according to the current leaf \n    order in Z.\n    \n    This is because when everything is sorted each cluster (including\n    singletons) can be defined by its range over (0...n_points).\n\n    This is used extensively to loop efficiently over the various arrays in the \n    algorithm.\n\n    '
    ...

def squareform(X, force, checks) -> typing.Any:
    "\n    Convert a vector-form distance vector to a square-form distance\n    matrix, and vice-versa.\n\n    Parameters\n    ----------\n    X : ndarray\n        Either a condensed or redundant distance matrix.\n    force : str, optional\n        As with MATLAB(TM), if force is equal to ``'tovector'`` or\n        ``'tomatrix'``, the input will be treated as a distance matrix or\n        distance vector respectively.\n    checks : bool, optional\n        If set to False, no checks will be made for matrix\n        symmetry nor zero diagonals. This is useful if it is known that\n        ``X - X.T1`` is small and ``diag(X)`` is close to zero.\n        These values are ignored any way so they do not disrupt the\n        squareform transformation.\n\n    Returns\n    -------\n    Y : ndarray\n        If a condensed distance matrix is passed, a redundant one is\n        returned, or if a redundant one is passed, a condensed distance\n        matrix is returned.\n\n    Notes\n    -----\n    1. ``v = squareform(X)``\n\n       Given a square n-by-n symmetric distance matrix ``X``,\n       ``v = squareform(X)`` returns a ``n * (n-1) / 2``\n       (i.e. binomial coefficient n choose 2) sized vector `v`\n       where :math:`v[{n \\choose 2} - {n-i \\choose 2} + (j-i-1)]`\n       is the distance between distinct points ``i`` and ``j``.\n       If ``X`` is non-square or asymmetric, an error is raised.\n\n    2. ``X = squareform(v)``\n\n       Given a ``n * (n-1) / 2`` sized vector ``v``\n       for some integer ``n >= 1`` encoding distances as described,\n       ``X = squareform(v)`` returns a n-by-n distance matrix ``X``.\n       The ``X[i, j]`` and ``X[j, i]`` values are set to\n       :math:`v[{n \\choose 2} - {n-i \\choose 2} + (j-i-1)]`\n       and all diagonal elements are zero.\n\n    In SciPy 0.19.0, ``squareform`` stopped casting all input types to\n    float64, and started returning arrays of the same dtype as the input.\n\n    "
    ...

def __getattr__(name) -> typing.Any:
    ...




from contextlib import contextmanager
import sys
import warnings
import re
import functools
import os

__all__ = ["all_warnings", "expected_warnings", "warn"]

# A version of `warnings.warn` with a default stacklevel of 2.
# functool is used so as not to increase the call stack accidentally
warn = ...

@contextmanager
def all_warnings(): ...
@contextmanager
def expected_warnings(matching): ...



from typing import Any, Callable, ClassVar, Literal, TypeVar
from ._ball_tree import BallTree
from ._base import KNeighborsMixin, RadiusNeighborsMixin, NeighborsBase
from numpy import ndarray
from ..utils._param_validation import StrOptions as StrOptions
from ..base import TransformerMixin, ClassNamePrefixFeaturesOutMixin
from ._unsupervised import NearestNeighbors
from scipy.sparse import spmatrix
from .._typing import MatrixLike, Int, Float
from ..utils.validation import check_is_fitted as check_is_fitted

RadiusNeighborsTransformer_Self = TypeVar(
    "RadiusNeighborsTransformer_Self", bound="RadiusNeighborsTransformer"
)
KNeighborsTransformer_Self = TypeVar(
    "KNeighborsTransformer_Self", bound="KNeighborsTransformer"
)


def kneighbors_graph(
    X: MatrixLike | BallTree | NearestNeighbors,
    n_neighbors: Int,
    *,
    mode: Literal["connectivity", "distance", "connectivity"] = "connectivity",
    metric: str = "minkowski",
    p: Int = 2,
    metric_params: None | dict = None,
    include_self: str | bool = False,
    n_jobs: None | Int = None,
) -> spmatrix:
    ...


def radius_neighbors_graph(
    X: MatrixLike | BallTree,
    radius: Float,
    *,
    mode: Literal["connectivity", "distance", "connectivity"] = "connectivity",
    metric: str = "minkowski",
    p: Int = 2,
    metric_params: None | dict = None,
    include_self: str | bool = False,
    n_jobs: None | Int = None,
):
    ...


class KNeighborsTransformer(
    ClassNamePrefixFeaturesOutMixin, KNeighborsMixin, TransformerMixin, NeighborsBase
):
    n_samples_fit_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    effective_metric_params_: dict = ...
    effective_metric_: str | Callable = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        mode: Literal["distance", "connectivity", "distance"] = "distance",
        n_neighbors: Int = 5,
        algorithm: Literal["auto", "ball_tree", "kd_tree", "brute", "auto"] = "auto",
        leaf_size: Int = 30,
        metric: str | Callable = "minkowski",
        p: Int = 2,
        metric_params: None | dict = None,
        n_jobs: None | Int = None,
    ) -> None:
        ...

    def fit(
        self: KNeighborsTransformer_Self, X: MatrixLike, y: Any = None
    ) -> KNeighborsTransformer_Self:
        ...

    def transform(self, X: MatrixLike) -> spmatrix:
        ...

    def fit_transform(self, X: MatrixLike, y: Any = None) -> spmatrix:
        ...


class RadiusNeighborsTransformer(
    ClassNamePrefixFeaturesOutMixin,
    RadiusNeighborsMixin,
    TransformerMixin,
    NeighborsBase,
):
    n_samples_fit_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    effective_metric_params_: dict = ...
    effective_metric_: str | Callable = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        mode: Literal["distance", "connectivity", "distance"] = "distance",
        radius: Float = 1.0,
        algorithm: Literal["auto", "ball_tree", "kd_tree", "brute", "auto"] = "auto",
        leaf_size: Int = 30,
        metric: str | Callable = "minkowski",
        p: Int = 2,
        metric_params: None | dict = None,
        n_jobs: None | Int = None,
    ) -> None:
        ...

    def fit(
        self: RadiusNeighborsTransformer_Self, X: MatrixLike, y: Any = None
    ) -> RadiusNeighborsTransformer_Self:
        ...

    def transform(self, X: MatrixLike):
        ...

    def fit_transform(self, X: MatrixLike, y: Any = None):
        ...



from enum import Enum

class _AutoStringNameEnum(Enum):
    def __hash__(self) -> int: ...

class JoinStyle(str, _AutoStringNameEnum):

    miter: JoinStyle 
    round: JoinStyle 
    bevel: JoinStyle 
    @staticmethod
    def demo() -> None: ...

class CapStyle(str, _AutoStringNameEnum):
    butt: CapStyle
    projecting: CapStyle
    round: CapStyle
    @staticmethod
    def demo() -> None: ...



from typing import ClassVar, Literal
from ..exceptions import (
    DataConversionWarning as DataConversionWarning,
    EfficiencyWarning as EfficiencyWarning,
)
from joblib import effective_n_jobs as effective_n_jobs
from ._ball_tree import BallTree as BallTree
from ..metrics import pairwise_distances_chunked as pairwise_distances_chunked
from ..utils.fixes import parse_version as parse_version, sp_version as sp_version
from ._kd_tree import KDTree as KDTree
from scipy.sparse import csr_matrix as csr_matrix, issparse as issparse, spmatrix
from ..utils.parallel import delayed as delayed, Parallel as Parallel
from ..metrics.pairwise import (
    PAIRWISE_DISTANCE_FUNCTIONS as PAIRWISE_DISTANCE_FUNCTIONS,
)
from ..utils.validation import (
    check_is_fitted as check_is_fitted,
    check_non_negative as check_non_negative,
)
from abc import ABCMeta, abstractmethod
from numpy import ndarray
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from numbers import Integral as Integral, Real as Real
from functools import partial as partial
from ..base import BaseEstimator, MultiOutputMixin, is_classifier as is_classifier
from ..utils import check_array as check_array, gen_even_slices as gen_even_slices
from ..metrics._pairwise_distances_reduction import (
    ArgKmin as ArgKmin,
    RadiusNeighbors as RadiusNeighbors,
)
from ..utils.multiclass import (
    check_classification_targets as check_classification_targets,
)
from .._typing import MatrixLike, Float, Int, ArrayLike

# Authors: Jake Vanderplas <vanderplas@astro.washington.edu>
#          Fabian Pedregosa <fabian.pedregosa@inria.fr>
#          Alexandre Gramfort <alexandre.gramfort@inria.fr>
#          Sparseness support by Lars Buitinck
#          Multi-output support by Arnaud Joly <a.joly@ulg.ac.be>
#
# License: BSD 3 clause (C) INRIA, University of Amsterdam
import itertools

import warnings
import numbers

import numpy as np

VALID_METRICS = ...

VALID_METRICS_SPARSE = ...


def sort_graph_by_row_values(
    graph: MatrixLike, copy: bool = False, warn_when_not_sorted: bool = True
) -> spmatrix:
    ...


class NeighborsBase(MultiOutputMixin, BaseEstimator, metaclass=ABCMeta):

    _parameter_constraints: ClassVar[dict] = ...

    @abstractmethod
    def __init__(
        self,
        n_neighbors: None | int = None,
        radius: None | Float = None,
        algorithm: str = "auto",
        leaf_size: int = 30,
        metric: str = "minkowski",
        p: None | int = 2,
        metric_params=None,
        n_jobs: None | int = None,
    ) -> None:
        ...


class KNeighborsMixin:
    def kneighbors(
        self,
        X: None | MatrixLike = None,
        n_neighbors: None | Int = None,
        return_distance: bool = True,
    ) -> ndarray | tuple[ndarray, ndarray]:
        ...

    def kneighbors_graph(
        self,
        X: None | MatrixLike = None,
        n_neighbors: None | Int = None,
        mode: Literal["connectivity", "connectivity", "distance"] = "connectivity",
    ) -> spmatrix:
        ...


class RadiusNeighborsMixin:
    def radius_neighbors(
        self,
        X: None | MatrixLike | list[ndarray] = None,
        radius: None | Float = None,
        return_distance: bool = True,
        sort_results: bool = False,
    ) -> ndarray | tuple[ndarray, ndarray]:
        ...

    def radius_neighbors_graph(
        self,
        X: None | MatrixLike | ArrayLike = None,
        radius: None | Float = None,
        mode: Literal["connectivity", "distance", "connectivity"] = "connectivity",
        sort_results: bool = False,
    ):
        ...



from typing import Any

from django.db import models

class FlatPage(models.Model):
    url: models.CharField[Any] = ...
    title: models.CharField[Any] = ...
    content: models.TextField[Any] = ...
    enable_comments: models.BooleanField[Any] = ...
    template_name: models.CharField[Any] = ...
    registration_required: models.BooleanField[Any] = ...
    sites: models.ManyToManyField[Any, Any] = ...
    def get_absolute_url(self) -> str: ...



from numpy import dtype
from numpy.typing import ArrayLike, NDArray

#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Note: we use functions (e.g. sin) from math module because they're faster

import math
import numpy as np

def translate(offset: ArrayLike, dtype: np.dtype | None = None) -> NDArray: ...
def scale(s: ArrayLike, dtype: np.dtype | None = None) -> NDArray: ...
def rotate(angle: float, axis: NDArray, dtype=None) -> NDArray: ...
def ortho(left: float, right: float, bottom: float, top: float, znear: float, zfar: float) -> NDArray: ...
def frustum(left: float, right: float, bottom: float, top: float, znear: float, zfar: float) -> NDArray: ...
def perspective(fovy: float, aspect: float, znear: float, zfar: float) -> NDArray: ...
def affine_map(points1, points2): ...



import numpy as np
from scipy.special import ellipkinc as ellip_F, ellipeinc as ellip_E

def ellipsoid(a: float, b: float, c: float, spacing=..., levelset: bool = False): ...
def ellipsoid_stats(a: float, b: float, c: float) -> tuple[float, float]: ...



from typing import Mapping

import itertools
from operator import itemgetter
from ...classes.graph import Graph

# Define the default maximum flow function to use in all flow based
# connectivity algorithms.
from ...algorithms.flow import (
    boykov_kolmogorov,
    build_residual_network,
    dinitz,
    edmonds_karp,
    shortest_augmenting_path,
)

default_flow_func = ...

from .utils import build_auxiliary_edge_connectivity, build_auxiliary_node_connectivity

__all__ = [
    "average_node_connectivity",
    "local_node_connectivity",
    "node_connectivity",
    "local_edge_connectivity",
    "edge_connectivity",
    "all_pairs_node_connectivity",
]

def local_node_connectivity(
    G: Graph, s, t, flow_func=None, auxiliary=None, residual=None, cutoff=None
): ...
def node_connectivity(G: Graph, s=None, t=None, flow_func=None): ...
def average_node_connectivity(G: Graph, flow_func=None) -> float: ...
def all_pairs_node_connectivity(G: Graph, nbunch=None, flow_func=None) -> Mapping: ...
def local_edge_connectivity(
    G: Graph, s, t, flow_func=None, auxiliary=None, residual=None, cutoff=None
): ...
def edge_connectivity(G: Graph, s=None, t=None, flow_func=None, cutoff=None): ...



from typing import Any, ClassVar, TypeVar
from numpy import ndarray
from ..utils._param_validation import Interval as Interval
from numbers import Real as Real, Integral as Integral
from .._config import config_context as config_context
from .._typing import MatrixLike, Float, Int
from ..utils import check_array as check_array
from . import empirical_covariance as empirical_covariance, EmpiricalCovariance

ShrunkCovariance_Self = TypeVar("ShrunkCovariance_Self", bound="ShrunkCovariance")
LedoitWolf_Self = TypeVar("LedoitWolf_Self", bound="LedoitWolf")
OAS_Self = TypeVar("OAS_Self", bound="OAS")


# Author: Alexandre Gramfort <alexandre.gramfort@inria.fr>
#         Gael Varoquaux <gael.varoquaux@normalesup.org>
#         Virgile Fritsch <virgile.fritsch@inria.fr>
#
# License: BSD 3 clause

# avoid division truncation
import warnings
import numpy as np


# ShrunkCovariance estimator


def shrunk_covariance(emp_cov: MatrixLike, shrinkage: Float = 0.1) -> ndarray:
    ...


class ShrunkCovariance(EmpiricalCovariance):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    precision_: ndarray = ...
    location_: ndarray = ...
    covariance_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        store_precision: bool = True,
        assume_centered: bool = False,
        shrinkage: Float = 0.1
    ) -> None:
        ...

    def fit(
        self: ShrunkCovariance_Self, X: MatrixLike, y: Any = None
    ) -> ShrunkCovariance_Self:
        ...


# Ledoit-Wolf estimator


def ledoit_wolf_shrinkage(
    X: MatrixLike, assume_centered: bool = False, block_size: Int = 1000
) -> Float:
    ...


def ledoit_wolf(
    X: MatrixLike, *, assume_centered: bool = False, block_size: Int = 1000
) -> tuple[ndarray, Float] | tuple[ndarray, float]:
    ...


class LedoitWolf(EmpiricalCovariance):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    shrinkage_: float = ...
    precision_: ndarray = ...
    location_: ndarray = ...
    covariance_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        store_precision: bool = True,
        assume_centered: bool = False,
        block_size: Int = 1000
    ) -> None:
        ...

    def fit(self: LedoitWolf_Self, X: MatrixLike, y: Any = None) -> LedoitWolf_Self:
        ...


# OAS estimator
def oas(
    X: MatrixLike, *, assume_centered: bool = False
) -> tuple[ndarray, Float] | tuple[ndarray, float]:
    ...


class OAS(EmpiricalCovariance):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    shrinkage_: float = ...
    precision_: ndarray = ...
    location_: ndarray = ...
    covariance_: ndarray = ...

    def fit(self: OAS_Self, X: MatrixLike, y: Any = None) -> OAS_Self:
        ...



from numpy.typing import NDArray

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np

from vispy.color.color_array import Color

from .line import LineVisual
from ..color import ColorArray
from ..color.colormap import _normalize, get_colormap
from ..geometry.isocurve import isocurve

class IsocurveVisual(LineVisual):
    def __init__(
        self,
        data: NDArray | None = None,
        levels: NDArray | None = None,
        color_lev: Color | str | tuple | list | np.ndarray | None = None,
        clim: tuple | None = None,
        **kwargs,
    ): ...
    @property
    def levels(self): ...
    @levels.setter
    def levels(self, levels): ...
    @property
    def color(self): ...
    @color.setter
    def color(self, color): ...
    def set_data(self, data: NDArray): ...
    def _get_verts_and_connect(self, paths): ...
    def _compute_iso_line(self): ...
    def _compute_iso_color(self): ...
    def _levels_to_colors(self): ...
    def _prepare_draw(self, view): ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.sparse.csgraph._traversal, version: unspecified
import typing
import builtins as _mod_builtins
import numpy as _mod_numpy
import scipy.sparse.csr as _mod_scipy_sparse_csr

DTYPE = _mod_numpy.float64
ITYPE = _mod_numpy.int32
__doc__: str
__file__: str
__name__: str
__package__: str
__test__: dict
def breadth_first_order(csgraph, i_start, directed=..., return_predecessors=...) -> typing.Any:
    '\n    breadth_first_order(csgraph, i_start, directed=True, return_predecessors=True)\n\n    Return a breadth-first ordering starting with specified node.\n\n    Note that a breadth-first order is not unique, but the tree which it\n    generates is unique.\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    csgraph : array_like or sparse matrix\n        The N x N compressed sparse graph.  The input csgraph will be\n        converted to csr format for the calculation.\n    i_start : int\n        The index of starting node.\n    directed : bool, optional\n        If True (default), then operate on a directed graph: only\n        move from point i to point j along paths csgraph[i, j].\n        If False, then find the shortest path on an undirected graph: the\n        algorithm can progress from point i to j along csgraph[i, j] or\n        csgraph[j, i].\n    return_predecessors : bool, optional\n        If True (default), then return the predecesor array (see below).\n\n    Returns\n    -------\n    node_array : ndarray, one dimension\n        The breadth-first list of nodes, starting with specified node.  The\n        length of node_array is the number of nodes reachable from the\n        specified node.\n    predecessors : ndarray, one dimension\n        Returned only if return_predecessors is True.\n        The length-N list of predecessors of each node in a breadth-first\n        tree.  If node i is in the tree, then its parent is given by\n        predecessors[i]. If node i is not in the tree (and for the parent\n        node) then predecessors[i] = -9999.\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix\n    >>> from scipy.sparse.csgraph import breadth_first_order\n\n    >>> graph = [\n    ... [0, 1 , 2, 0],\n    ... [0, 0, 0, 1],\n    ... [2, 0, 0, 3],\n    ... [0, 0, 0, 0]\n    ... ]\n    >>> graph = csr_matrix(graph)\n    >>> print(graph)\n      (0, 1)    1\n      (0, 2)    2\n      (1, 3)    1\n      (2, 0)    2\n      (2, 3)    3\n\n    >>> breadth_first_order(graph,0)\n    (array([0, 1, 2, 3], dtype=int32), array([-9999,     0,     0,     1], dtype=int32))\n\n    '
    ...

def breadth_first_tree(csgraph, i_start, directed=...) -> typing.Any:
    '\n    breadth_first_tree(csgraph, i_start, directed=True)\n\n    Return the tree generated by a breadth-first search\n\n    Note that a breadth-first tree from a specified node is unique.\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    csgraph : array_like or sparse matrix\n        The N x N matrix representing the compressed sparse graph.  The input\n        csgraph will be converted to csr format for the calculation.\n    i_start : int\n        The index of starting node.\n    directed : bool, optional\n        If True (default), then operate on a directed graph: only\n        move from point i to point j along paths csgraph[i, j].\n        If False, then find the shortest path on an undirected graph: the\n        algorithm can progress from point i to j along csgraph[i, j] or\n        csgraph[j, i].\n\n    Returns\n    -------\n    cstree : csr matrix\n        The N x N directed compressed-sparse representation of the breadth-\n        first tree drawn from csgraph, starting at the specified node.\n\n    Examples\n    --------\n    The following example shows the computation of a depth-first tree\n    over a simple four-component graph, starting at node 0::\n\n         input graph          breadth first tree from (0)\n\n             (0)                         (0)\n            /   \\                       /   \\\n           3     8                     3     8\n          /       \\                   /       \\\n        (3)---5---(1)               (3)       (1)\n          \\       /                           /\n           6     2                           2\n            \\   /                           /\n             (2)                         (2)\n\n    In compressed sparse representation, the solution looks like this:\n\n    >>> from scipy.sparse import csr_matrix\n    >>> from scipy.sparse.csgraph import breadth_first_tree\n    >>> X = csr_matrix([[0, 8, 0, 3],\n    ...                 [0, 0, 2, 5],\n    ...                 [0, 0, 0, 6],\n    ...                 [0, 0, 0, 0]])\n    >>> Tcsr = breadth_first_tree(X, 0, directed=False)\n    >>> Tcsr.toarray().astype(int)\n    array([[0, 8, 0, 3],\n           [0, 0, 2, 0],\n           [0, 0, 0, 0],\n           [0, 0, 0, 0]])\n\n    Note that the resulting graph is a Directed Acyclic Graph which spans\n    the graph.  A breadth-first tree from a given node is unique.\n    '
    ...

def connected_components(csgraph, directed=..., connection=..., return_labels=...) -> typing.Any:
    '\n    connected_components(csgraph, directed=True, connection=\'weak\',\n                         return_labels=True)\n\n    Analyze the connected components of a sparse graph\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    csgraph : array_like or sparse matrix\n        The N x N matrix representing the compressed sparse graph.  The input\n        csgraph will be converted to csr format for the calculation.\n    directed : bool, optional\n        If True (default), then operate on a directed graph: only\n        move from point i to point j along paths csgraph[i, j].\n        If False, then find the shortest path on an undirected graph: the\n        algorithm can progress from point i to j along csgraph[i, j] or\n        csgraph[j, i].\n    connection : str, optional\n        [\'weak\'|\'strong\'].  For directed graphs, the type of connection to\n        use.  Nodes i and j are strongly connected if a path exists both\n        from i to j and from j to i. A directed graph is weakly connected\n        if replacing all of its directed edges with undirected edges produces\n        a connected (undirected) graph. If directed == False, this keyword\n        is not referenced.\n    return_labels : bool, optional\n        If True (default), then return the labels for each of the connected\n        components.\n\n    Returns\n    -------\n    n_components: int\n        The number of connected components.\n    labels: ndarray\n        The length-N array of labels of the connected components.\n\n    References\n    ----------\n    .. [1] D. J. Pearce, "An Improved Algorithm for Finding the Strongly\n           Connected Components of a Directed Graph", Technical Report, 2005\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix\n    >>> from scipy.sparse.csgraph import connected_components\n\n    >>> graph = [\n    ... [ 0, 1 , 1, 0 , 0 ],\n    ... [ 0, 0 , 1 , 0 ,0 ],\n    ... [ 0, 0, 0, 0, 0],\n    ... [0, 0 , 0, 0, 1],\n    ... [0, 0, 0, 0, 0]\n    ... ]\n    >>> graph = csr_matrix(graph)\n    >>> print(graph)\n      (0, 1)\t1\n      (0, 2)\t1\n      (1, 2)\t1\n      (3, 4)\t1\n\n    >>> n_components, labels = connected_components(csgraph=graph, directed=False, return_labels=True)\n    >>> n_components\n    2\n    >>> labels\n    array([0, 0, 0, 1, 1], dtype=int32)\n\n    '
    ...

csr_matrix = _mod_scipy_sparse_csr.csr_matrix
def depth_first_order(csgraph, i_start, directed=..., return_predecessors=...) -> typing.Any:
    '\n    depth_first_order(csgraph, i_start, directed=True, return_predecessors=True)\n\n    Return a depth-first ordering starting with specified node.\n\n    Note that a depth-first order is not unique.  Furthermore, for graphs\n    with cycles, the tree generated by a depth-first search is not\n    unique either.\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    csgraph : array_like or sparse matrix\n        The N x N compressed sparse graph.  The input csgraph will be\n        converted to csr format for the calculation.\n    i_start : int\n        The index of starting node.\n    directed : bool, optional\n        If True (default), then operate on a directed graph: only\n        move from point i to point j along paths csgraph[i, j].\n        If False, then find the shortest path on an undirected graph: the\n        algorithm can progress from point i to j along csgraph[i, j] or\n        csgraph[j, i].\n    return_predecessors : bool, optional\n        If True (default), then return the predecesor array (see below).\n\n    Returns\n    -------\n    node_array : ndarray, one dimension\n        The depth-first list of nodes, starting with specified node.  The\n        length of node_array is the number of nodes reachable from the\n        specified node.\n    predecessors : ndarray, one dimension\n        Returned only if return_predecessors is True.\n        The length-N list of predecessors of each node in a depth-first\n        tree.  If node i is in the tree, then its parent is given by\n        predecessors[i]. If node i is not in the tree (and for the parent\n        node) then predecessors[i] = -9999.\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix\n    >>> from scipy.sparse.csgraph import depth_first_order\n\n    >>> graph = [\n    ... [0, 1 , 2, 0],\n    ... [0, 0, 0, 1],\n    ... [2, 0, 0, 3],\n    ... [0, 0, 0, 0]\n    ... ]\n    >>> graph = csr_matrix(graph)\n    >>> print(graph)\n      (0, 1)\t1\n      (0, 2)\t2\n      (1, 3)\t1\n      (2, 0)\t2\n      (2, 3)\t3\n\n    >>> depth_first_order(graph,0)\n    (array([0, 1, 3, 2], dtype=int32), array([-9999,     0,     0,     1], dtype=int32))\n\n    '
    ...

def depth_first_tree(csgraph, i_start, directed=...) -> typing.Any:
    '\n    depth_first_tree(csgraph, i_start, directed=True)\n\n    Return a tree generated by a depth-first search.\n\n    Note that a tree generated by a depth-first search is not unique:\n    it depends on the order that the children of each node are searched.\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    csgraph : array_like or sparse matrix\n        The N x N matrix representing the compressed sparse graph.  The input\n        csgraph will be converted to csr format for the calculation.\n    i_start : int\n        The index of starting node.\n    directed : bool, optional\n        If True (default), then operate on a directed graph: only\n        move from point i to point j along paths csgraph[i, j].\n        If False, then find the shortest path on an undirected graph: the\n        algorithm can progress from point i to j along csgraph[i, j] or\n        csgraph[j, i].\n\n    Returns\n    -------\n    cstree : csr matrix\n        The N x N directed compressed-sparse representation of the depth-\n        first tree drawn from csgraph, starting at the specified node.\n\n    Examples\n    --------\n    The following example shows the computation of a depth-first tree\n    over a simple four-component graph, starting at node 0::\n\n         input graph           depth first tree from (0)\n\n             (0)                         (0)\n            /   \\                           \\\n           3     8                           8\n          /       \\                           \\\n        (3)---5---(1)               (3)       (1)\n          \\       /                   \\       /\n           6     2                     6     2\n            \\   /                       \\   /\n             (2)                         (2)\n\n    In compressed sparse representation, the solution looks like this:\n\n    >>> from scipy.sparse import csr_matrix\n    >>> from scipy.sparse.csgraph import depth_first_tree\n    >>> X = csr_matrix([[0, 8, 0, 3],\n    ...                 [0, 0, 2, 5],\n    ...                 [0, 0, 0, 6],\n    ...                 [0, 0, 0, 0]])\n    >>> Tcsr = depth_first_tree(X, 0, directed=False)\n    >>> Tcsr.toarray().astype(int)\n    array([[0, 8, 0, 0],\n           [0, 0, 2, 0],\n           [0, 0, 0, 6],\n           [0, 0, 0, 0]])\n\n    Note that the resulting graph is a Directed Acyclic Graph which spans\n    the graph.  Unlike a breadth-first tree, a depth-first tree of a given\n    graph is not unique if the graph contains cycles.  If the above solution\n    had begun with the edge connecting nodes 0 and 3, the result would have\n    been different.\n    '
    ...

def isspmatrix(x) -> typing.Any:
    'Is x of a sparse matrix type?\n\n    Parameters\n    ----------\n    x\n        object to check for being a sparse matrix\n\n    Returns\n    -------\n    bool\n        True if x is a sparse matrix, False otherwise\n\n    Notes\n    -----\n    issparse and isspmatrix are aliases for the same function.\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix, isspmatrix\n    >>> isspmatrix(csr_matrix([[5]]))\n    True\n\n    >>> from scipy.sparse import isspmatrix\n    >>> isspmatrix(5)\n    False\n    '
    ...

def isspmatrix_csc(x) -> typing.Any:
    'Is x of csc_matrix type?\n\n    Parameters\n    ----------\n    x\n        object to check for being a csc matrix\n\n    Returns\n    -------\n    bool\n        True if x is a csc matrix, False otherwise\n\n    Examples\n    --------\n    >>> from scipy.sparse import csc_matrix, isspmatrix_csc\n    >>> isspmatrix_csc(csc_matrix([[5]]))\n    True\n\n    >>> from scipy.sparse import csc_matrix, csr_matrix, isspmatrix_csc\n    >>> isspmatrix_csc(csr_matrix([[5]]))\n    False\n    '
    ...

def isspmatrix_csr(x) -> typing.Any:
    'Is x of csr_matrix type?\n\n    Parameters\n    ----------\n    x\n        object to check for being a csr matrix\n\n    Returns\n    -------\n    bool\n        True if x is a csr matrix, False otherwise\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix, isspmatrix_csr\n    >>> isspmatrix_csr(csr_matrix([[5]]))\n    True\n\n    >>> from scipy.sparse import csc_matrix, csr_matrix, isspmatrix_csc\n    >>> isspmatrix_csr(csc_matrix([[5]]))\n    False\n    '
    ...

def reconstruct_path(csgraph, predecessors, directed=...) -> typing.Any:
    '\n    reconstruct_path(csgraph, predecessors, directed=True)\n\n    Construct a tree from a graph and a predecessor list.\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    csgraph : array_like or sparse matrix\n        The N x N matrix representing the directed or undirected graph\n        from which the predecessors are drawn.\n    predecessors : array_like, one dimension\n        The length-N array of indices of predecessors for the tree.  The\n        index of the parent of node i is given by predecessors[i].\n    directed : bool, optional\n        If True (default), then operate on a directed graph: only move from\n        point i to point j along paths csgraph[i, j].\n        If False, then operate on an undirected graph: the algorithm can\n        progress from point i to j along csgraph[i, j] or csgraph[j, i].\n\n    Returns\n    -------\n    cstree : csr matrix\n        The N x N directed compressed-sparse representation of the tree drawn\n        from csgraph which is encoded by the predecessor list.\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix\n    >>> from scipy.sparse.csgraph import reconstruct_path\n\n    >>> graph = [\n    ... [0, 1 , 2, 0],\n    ... [0, 0, 0, 1],\n    ... [0, 0, 0, 3],\n    ... [0, 0, 0, 0]\n    ... ]\n    >>> graph = csr_matrix(graph)\n    >>> print(graph)\n      (0, 1)\t1\n      (0, 2)\t2\n      (1, 3)\t1\n      (2, 3)\t3\n\n    >>> pred = np.array([-9999, 0, 0, 1], dtype=np.int32)\n\n    >>> cstree = reconstruct_path(csgraph=graph, predecessors=pred, directed=False)\n    >>> cstree.todense()\n    matrix([[ 0.,  1.,  2.,  0.],\n            [ 0.,  0.,  0.,  1.],\n            [ 0.,  0.,  0.,  0.],\n            [ 0.,  0.,  0.,  0.]])\n\n    '
    ...

def validate_graph(csgraph, directed, dtype, csr_output, dense_output, copy_if_dense, copy_if_sparse, null_value_in, null_value_out, infinity_null, nan_null) -> typing.Any:
    'Routine for validation and conversion of csgraph inputs'
    ...

def __getattr__(name) -> typing.Any:
    ...




# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.linalg.cython_blas, version: unspecified
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
__pyx_capi__: dict
def __pyx_unpickle_Enum() -> typing.Any:
    ...

__test__: dict
def _test_cdotc() -> typing.Any:
    ...

def _test_cdotu() -> typing.Any:
    ...

def _test_dasum() -> typing.Any:
    ...

def _test_ddot() -> typing.Any:
    ...

def _test_dgemm() -> typing.Any:
    ...

def _test_dnrm2() -> typing.Any:
    ...

def _test_dzasum() -> typing.Any:
    ...

def _test_dznrm2() -> typing.Any:
    ...

def _test_icamax() -> typing.Any:
    ...

def _test_idamax() -> typing.Any:
    ...

def _test_isamax() -> typing.Any:
    ...

def _test_izamax() -> typing.Any:
    ...

def _test_sasum() -> typing.Any:
    ...

def _test_scasum() -> typing.Any:
    ...

def _test_scnrm2() -> typing.Any:
    ...

def _test_sdot() -> typing.Any:
    ...

def _test_snrm2() -> typing.Any:
    ...

def _test_zdotc() -> typing.Any:
    ...

def _test_zdotu() -> typing.Any:
    ...

def __getattr__(name) -> typing.Any:
    ...




# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy._lib._ccallback_c, version: unspecified
import typing
import builtins as _mod_builtins
import ctypes as _mod_ctypes

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
__pyx_capi__: dict
__test__: dict
def check_capsule(item) -> typing.Any:
    '\n    check_capsule(item)\n\n    Return True if the given object is a PyCapsule.\n    '
    ...

def get_capsule_signature() -> typing.Any:
    ...

def get_raw_capsule(ptr, name, context) -> typing.Any:
    '\n    get_raw_capsule(ptr, name, context)\n\n    Create a new PyCapsule with given pointer, name, and context.\n\n    Parameters\n    ----------\n    ptr : {PyCapsule, int}\n        Memory address of the pointer.\n    name : str\n        Python string containing the signature.\n    context : {PyCapsule, int}\n        Memory address of the context.\n        If NULL and ptr is a PyCapsule, use the one from the context of ptr.\n\n    '
    ...

idx: int
def plus1_ctypes() -> typing.Any:
    ...

plus1_t = _mod_ctypes.CFunctionType
def plus1b_ctypes() -> typing.Any:
    ...

plus1b_t = _mod_ctypes.CFunctionType
def plus1bc_ctypes() -> typing.Any:
    ...

plus1bc_t = _mod_ctypes.CFunctionType
sig: tuple
sigs: list
def sine_ctypes() -> typing.Any:
    ...

sine_t = _mod_ctypes.CFunctionType
def test_call_cython() -> typing.Any:
    '\n    Implementation of a caller routine in Cython\n    '
    ...

def __getattr__(name) -> typing.Any:
    ...




from numpy.typing import NDArray
import math

import numpy as np
import scipy.ndimage as ndi
from scipy import spatial

from .._shared.filters import gaussian
from .._shared.utils import _supported_float_type, check_nD
from ..transform import integral_image

from .peak import peak_local_max

# This basic blob detection algorithm is based on:
# http://www.cs.utah.edu/~jfishbau/advimproc/project1/ (04.04.2013)
# Theory behind: https://en.wikipedia.org/wiki/Blob_detection (04.04.2013)

def _compute_disk_overlap(d, r1, r2): ...
def _compute_sphere_overlap(d, r1, r2): ...
def _blob_overlap(blob1, blob2, *, sigma_dim=1): ...
def _prune_blobs(blobs_array, overlap, *, sigma_dim=1): ...
def _format_exclude_border(img_ndim, exclude_border): ...
def blob_dog(
    image: NDArray,
    min_sigma=1,
    max_sigma=50,
    sigma_ratio: float = 1.6,
    threshold: float | None = 0.5,
    overlap: float = 0.5,
    *,
    threshold_rel: float | None = None,
    exclude_border=False
): ...
def blob_log(
    image: NDArray,
    min_sigma=1,
    max_sigma=50,
    num_sigma: int = 10,
    threshold: float | None = 0.2,
    overlap: float = 0.5,
    log_scale: bool = False,
    *,
    threshold_rel: float | None = None,
    exclude_border=False
): ...
def blob_doh(
    image,
    min_sigma: float = 1,
    max_sigma: float = 30,
    num_sigma: int = 10,
    threshold: float | None = 0.01,
    overlap: float = 0.5,
    log_scale: bool = False,
    *,
    threshold_rel: float | None = None
): ...



from typing import Mapping

from itertools import accumulate

from ..classes.graph import Graph
from ..utils import not_implemented_for

__all__ = ["rich_club_coefficient"]

def rich_club_coefficient(G: Graph, normalized=True, Q=100, seed=None) -> Mapping: ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.interpolate._fitpack, version:  1.7 
import typing
import builtins as _mod_builtins

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
__version__: str
def _bispev(tx, ty, c, kx, ky, x, y, nux, nuy) -> typing.Any:
    ' [z,ier] = _bispev(tx,ty,c,kx,ky,x,y,nux,nuy)'
    ...

def _bspldismat(order, xk) -> typing.Any:
    'B = _bspldismat(order,xk)\nConstruct the kth derivative discontinuity jump constraint matrix \nfor spline fitting of order k given sample positions in xk.\n\nIf xk is an integer (N+1), then the result is equivalent to\nxk=arange(N+1)+x0 for any value of x0.   This produces the\ninteger-spaced matrix a bit faster.  If xk is a 2-tuple (N+1,dx)\nthen it produces the result as if the sample distance were dx'
    ...

def _bspleval(xx, xk, coef, k, deriv) -> typing.Any:
    'y = _bspleval(xx,xk,coef,k,{deriv (0)})\n\nThe spline is defined by the approximation interval xk[0] to xk[-1],\nthe length of xk (N+1), the order of the spline, k, and \nthe number of coeficients N+k.  The coefficients range from xk_{-K}\nto xk_{N-1} inclusive and are all the coefficients needed to define\nan arbitrary spline of order k, on the given approximation interval\n\nExtra knot points are internally added using knot-point symmetry \naround xk[0] and xk[-1]'
    ...

def _bsplmat(order, xk) -> typing.Any:
    'B = _bsplmat(order,xk)\nConstruct the constraint matrix for spline fitting of order k\ngiven sample positions in xk.\n\nIf xk is an integer (N+1), then the result is equivalent to\nxk=arange(N+1)+x0 for any value of x0.   This produces the\ninteger-spaced, or cardinal spline matrix a bit faster.'
    ...

def _curfit(x, y, w, xb, xe, k, iopt, s, t, nest, wrk, iwrk, per) -> typing.Any:
    ' [t,c,o] = _curfit(x,y,w,xb,xe,k,iopt,s,t,nest,wrk,iwrk,per)'
    ...

def _insert(iopt, t, c, k, x, m) -> typing.Any:
    ' [tt,cc,ier] = _insert(iopt,t,c,k,x,m)'
    ...

def _parcur(x, w, u, ub, ue, k, iopt, ipar, s, t, nest, wrk, iwrk, per) -> typing.Any:
    ' [t,c,o] = _parcur(x,w,u,ub,ue,k,iopt,ipar,s,t,nest,wrk,iwrk,per)'
    ...

def _spalde(t, c, k, x) -> typing.Any:
    ' [d,ier] = _spalde(t,c,k,x)'
    ...

def _spl_(x, nu, t, c, k, e) -> typing.Any:
    ' [y,ier] = _spl_(x,nu,t,c,k,e)'
    ...

def _splint(t, c, k, a, b) -> typing.Any:
    ' [aint,wrk] = _splint(t,c,k,a,b)'
    ...

def _sproot(t, c, k, mest) -> typing.Any:
    ' [z,ier] = _sproot(t,c,k,mest)'
    ...

def _surfit(x, y, z, w, xb, xe, yb, ye, kx, ky, iopt, s, eps, tx, ty, nxest, nyest, wrk, lwrk1, lwrk2) -> typing.Any:
    ' [tx,ty,c,o] = _surfit(x, y, z, w, xb, xe, yb, ye, kx,ky,iopt,s,eps,tx,ty,nxest,nyest,wrk,lwrk1,lwrk2)'
    ...

def __getattr__(name) -> typing.Any:
    ...




from .glm import (
    _GeneralizedLinearRegressor as _GeneralizedLinearRegressor,
    PoissonRegressor as PoissonRegressor,
    GammaRegressor as GammaRegressor,
    TweedieRegressor as TweedieRegressor,
)

# License: BSD 3 clause


__all__ = [
    "_GeneralizedLinearRegressor",
    "PoissonRegressor",
    "GammaRegressor",
    "TweedieRegressor",
]



from typing import (
    Any,
    Callable,
    ClassVar,
    Generic,
    Iterable,
    Iterator,
    Mapping,
    Sequence,
    TypeVar,
)
from numpy.random import RandomState
from itertools import product as product
from ..base import BaseEstimator
from ..exceptions import NotFittedError as NotFittedError
from ._split import BaseShuffleSplit
from ..utils.random import sample_without_replacement as sample_without_replacement
from ..metrics import check_scoring as check_scoring
from ._split import check_cv as check_cv
from scipy.sparse import spmatrix
from ..utils.parallel import delayed as delayed, Parallel as Parallel
from collections.abc import Mapping, Sequence, Iterable
from ..utils.validation import (
    indexable as indexable,
    check_is_fitted as check_is_fitted,
)
from scipy.stats import rankdata as rankdata
from collections import defaultdict as defaultdict
from abc import ABCMeta, abstractmethod
from numpy import ndarray
from functools import partial as partial, reduce as reduce
from ..base import is_classifier as is_classifier, clone as clone, MetaEstimatorMixin
from ..utils import check_random_state as check_random_state
from ..utils.metaestimators import available_if as available_if
from numpy.ma import MaskedArray as MaskedArray
from .._typing import Int, MatrixLike, ArrayLike, Float
from . import BaseCrossValidator

BaseSearchCV_Self = TypeVar("BaseSearchCV_Self", bound="BaseSearchCV")
BaseEstimatorT = TypeVar("BaseEstimatorT", bound=BaseEstimator, default=BaseEstimator, covariant=True)

import numbers
import operator
import time
import warnings

import numpy as np

__all__ = ["GridSearchCV", "ParameterGrid", "ParameterSampler", "RandomizedSearchCV"]


class ParameterGrid:
    def __init__(
        self, param_grid: Sequence[Mapping[str, Sequence]] | Mapping[str, Sequence]
    ) -> None:
        ...

    def __iter__(self) -> Iterator[dict[str, Any]]:
        ...

    def __len__(self) -> int:
        ...

    def __getitem__(self, ind: Int) -> dict[str, Any]:
        ...


class ParameterSampler:
    def __init__(
        self,
        param_distributions: dict,
        n_iter: Int,
        *,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...

    def __iter__(self):
        ...

    def __len__(self) -> int:
        ...


class BaseSearchCV(MetaEstimatorMixin, BaseEstimator, metaclass=ABCMeta):
    @abstractmethod
    def __init__(
        self,
        estimator,
        *,
        scoring=None,
        n_jobs=None,
        refit: bool = True,
        cv=None,
        verbose: int = 0,
        pre_dispatch: str = "2*n_jobs",
        error_score=...,
        return_train_score: bool = True,
    ) -> None:
        ...

    def score(
        self, X: list[str] | MatrixLike, y: None | MatrixLike | ArrayLike = None
    ) -> Float:
        ...

    def score_samples(self, X: Iterable) -> ndarray:
        ...

    def predict(self, X: ArrayLike) -> ndarray:
        ...

    def predict_proba(self, X: ArrayLike) -> ndarray:
        ...

    def predict_log_proba(self, X: ArrayLike) -> ndarray:
        ...

    def decision_function(self, X: ArrayLike) -> ndarray:
        ...

    def transform(self, X: ArrayLike) -> ndarray | spmatrix:
        ...

    def inverse_transform(self, Xt: ArrayLike) -> ndarray | spmatrix:
        ...

    def n_features_in_(self):
        ...

    def classes_(self):
        ...

    def fit(
        self: BaseSearchCV_Self,
        X: list[str] | MatrixLike,
        y: None | MatrixLike | ArrayLike = None,
        *,
        groups: None | ArrayLike = None,
        **fit_params,
    ) -> BaseSearchCV_Self:
        ...


class GridSearchCV(Generic[BaseEstimatorT], BaseSearchCV):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    classes_: ndarray = ...
    multimetric_: bool = ...
    refit_time_: float = ...
    n_splits_: int = ...
    scorer_: Callable | dict = ...
    best_index_: int = ...
    best_params_: dict = ...
    best_score_: float = ...
    best_estimator_: BaseEstimatorT = ...
    cv_results_: dict[str, ndarray] = ...

    _required_parameters: ClassVar[list] = ...

    def __init__(
        self,
        estimator: BaseEstimatorT,
        param_grid: Mapping | Sequence[dict],
        *,
        scoring: ArrayLike | None | tuple | Callable | Mapping | str = None,
        n_jobs: None | Int = None,
        refit: str | Callable | bool = True,
        cv: int | BaseCrossValidator | Iterable | None | BaseShuffleSplit = None,
        verbose: Int = 0,
        pre_dispatch: str | int = "2*n_jobs",
        error_score: str | Float = ...,
        return_train_score: bool = False,
    ) -> None:
        ...


class RandomizedSearchCV(BaseSearchCV):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    classes_: ndarray = ...
    multimetric_: bool = ...
    refit_time_: float = ...
    n_splits_: int = ...
    scorer_: Callable | dict = ...
    best_index_: int = ...
    best_params_: dict = ...
    best_score_: float = ...
    best_estimator_: BaseEstimator = ...
    cv_results_: dict[str, ndarray] = ...

    _required_parameters: ClassVar[list] = ...

    def __init__(
        self,
        estimator: BaseEstimator,
        param_distributions: Sequence[Mapping] | Mapping,
        *,
        n_iter: Int = 10,
        scoring: ArrayLike | None | tuple | Callable | Mapping | str = None,
        n_jobs: None | Int = None,
        refit: str | Callable | bool = True,
        cv: int | BaseCrossValidator | Iterable | None | BaseShuffleSplit = None,
        verbose: Int = 0,
        pre_dispatch: str | int = "2*n_jobs",
        random_state: RandomState | None | Int = None,
        error_score: str | Float = ...,
        return_train_score: bool = False,
    ) -> None:
        ...



"""
This type stub file was generated by pyright.
"""

import threading
from traitlets.config.configurable import LoggingConfigurable
from decorator import decorator
from IPython.utils.decorators import undoc
from traitlets import Integer

""" History related magics and functionality """
@undoc
class DummyDB:
    """Dummy DB that will act as a black hole for history.

    Only used in the absence of sqlite"""
    def execute(*args, **kwargs):
        ...
    
    def commit(self, *args, **kwargs): # -> None:
        ...
    
    def __enter__(self, *args, **kwargs): # -> None:
        ...
    
    def __exit__(self, *args, **kwargs): # -> None:
        ...
    


@decorator
def only_when_enabled(f, self, *a, **kw):
    """Decorator: return an empty list in the absence of sqlite."""
    ...

_SAVE_DB_SIZE = ...
@decorator
def catch_corrupt_db(f, self, *a, **kw):
    """A decorator which wraps HistoryAccessor method calls to catch errors from
    a corrupt SQLite database, move the old database out of the way, and create
    a new one.

    We avoid clobbering larger databases because this may be triggered due to filesystem issues,
    not just a corrupt file.
    """
    ...

class HistoryAccessorBase(LoggingConfigurable):
    """An abstract class for History Accessors """
    def get_tail(self, n=..., raw=..., output=..., include_latest=...):
        ...
    
    def search(self, pattern=..., raw=..., search_raw=..., output=..., n=..., unique=...):
        ...
    
    def get_range(self, session, start=..., stop=..., raw=..., output=...):
        ...
    
    def get_range_by_str(self, rangestr, raw=..., output=...):
        ...
    


class HistoryAccessor(HistoryAccessorBase):
    """Access the history database without adding to it.

    This is intended for use by standalone history tools. IPython shells use
    HistoryManager, below, which is a subclass of this."""
    _corrupt_db_counter = ...
    _corrupt_db_limit = ...
    hist_file = ...
    enabled = ...
    connection_options = ...
    db = ...
    def __init__(self, profile=..., hist_file=..., **traits) -> None:
        """Create a new history accessor.

        Parameters
        ----------
        profile : str
            The name of the profile from which to open history.
        hist_file : str
            Path to an SQLite history database stored by IPython. If specified,
            hist_file overrides profile.
        config : :class:`~traitlets.config.loader.Config`
            Config object. hist_file can also be set through this.
        """
        ...
    
    @catch_corrupt_db
    def init_db(self): # -> None:
        """Connect to the database, and create tables if necessary."""
        ...
    
    def writeout_cache(self): # -> None:
        """Overridden by HistoryManager to dump the cache before certain
        database lookups."""
        ...
    
    @only_when_enabled
    @catch_corrupt_db
    def get_session_info(self, session):
        """Get info about a session.

        Parameters
        ----------
        session : int
            Session number to retrieve.

        Returns
        -------
        session_id : int
            Session ID number
        start : datetime
            Timestamp for the start of the session.
        end : datetime
            Timestamp for the end of the session, or None if IPython crashed.
        num_cmds : int
            Number of commands run, or None if IPython crashed.
        remark : unicode
            A manually set description.
        """
        ...
    
    @catch_corrupt_db
    def get_last_session_id(self): # -> None:
        """Get the last session ID currently in the database.

        Within IPython, this should be the same as the value stored in
        :attr:`HistoryManager.session_number`.
        """
        ...
    
    @catch_corrupt_db
    def get_tail(self, n=..., raw=..., output=..., include_latest=...):
        """Get the last n lines from the history database.

        Parameters
        ----------
        n : int
            The number of lines to get
        raw, output : bool
            See :meth:`get_range`
        include_latest : bool
            If False (default), n+1 lines are fetched, and the latest one
            is discarded. This is intended to be used where the function
            is called by a user command, which it should not return.

        Returns
        -------
        Tuples as :meth:`get_range`
        """
        ...
    
    @catch_corrupt_db
    def search(self, pattern=..., raw=..., search_raw=..., output=..., n=..., unique=...):
        """Search the database using unix glob-style matching (wildcards
        * and ?).

        Parameters
        ----------
        pattern : str
            The wildcarded pattern to match when searching
        search_raw : bool
            If True, search the raw input, otherwise, the parsed input
        raw, output : bool
            See :meth:`get_range`
        n : None or int
            If an integer is given, it defines the limit of
            returned entries.
        unique : bool
            When it is true, return only unique entries.

        Returns
        -------
        Tuples as :meth:`get_range`
        """
        ...
    
    @catch_corrupt_db
    def get_range(self, session, start=..., stop=..., raw=..., output=...):
        """Retrieve input by session.

        Parameters
        ----------
        session : int
            Session number to retrieve.
        start : int
            First line to retrieve.
        stop : int
            End of line range (excluded from output itself). If None, retrieve
            to the end of the session.
        raw : bool
            If True, return untranslated input
        output : bool
            If True, attempt to include output. This will be 'real' Python
            objects for the current session, or text reprs from previous
            sessions if db_log_output was enabled at the time. Where no output
            is found, None is used.

        Returns
        -------
        entries
            An iterator over the desired lines. Each line is a 3-tuple, either
            (session, line, input) if output is False, or
            (session, line, (input, output)) if output is True.
        """
        ...
    
    def get_range_by_str(self, rangestr, raw=..., output=...):
        """Get lines of history from a string of ranges, as used by magic
        commands %hist, %save, %macro, etc.

        Parameters
        ----------
        rangestr : str
            A string specifying ranges, e.g. "5 ~2/1-4". If empty string is used,
            this will return everything from current session's history.

            See the documentation of :func:`%history` for the full details.

        raw, output : bool
            As :meth:`get_range`

        Returns
        -------
        Tuples as :meth:`get_range`
        """
        ...
    


class HistoryManager(HistoryAccessor):
    """A class to organize all history-related functionality in one place.
    """
    shell = ...
    input_hist_parsed = ...
    input_hist_raw = ...
    dir_hist = ...
    output_hist = ...
    output_hist_reprs = ...
    session_number = Integer()
    db_log_output = ...
    db_cache_size = Integer(0, help="Write to database every x commands (higher values save disk access & power).\n" "Values of 1 or less effectively disable caching.").tag(config=True)
    db_input_cache = ...
    db_output_cache = ...
    save_thread = ...
    save_flag = ...
    _i00 = ...
    _i = ...
    _ii = ...
    _iii = ...
    _exit_re = ...
    def __init__(self, shell=..., config=..., **traits) -> None:
        """Create a new history manager associated with a shell instance.
        """
        ...
    
    @only_when_enabled
    def new_session(self, conn=...): # -> None:
        """Get a new session number."""
        ...
    
    def end_session(self): # -> None:
        """Close the database session, filling in the end time and line count."""
        ...
    
    def name_session(self, name): # -> None:
        """Give the current session a name in the history database."""
        ...
    
    def reset(self, new_session=...): # -> None:
        """Clear the session history, releasing all object references, and
        optionally open a new session."""
        ...
    
    def get_session_info(self, session=...):
        """Get info about a session.

        Parameters
        ----------
        session : int
            Session number to retrieve. The current session is 0, and negative
            numbers count back from current session, so -1 is the previous session.

        Returns
        -------
        session_id : int
            Session ID number
        start : datetime
            Timestamp for the start of the session.
        end : datetime
            Timestamp for the end of the session, or None if IPython crashed.
        num_cmds : int
            Number of commands run, or None if IPython crashed.
        remark : unicode
            A manually set description.
        """
        ...
    
    @catch_corrupt_db
    def get_tail(self, n=..., raw=..., output=..., include_latest=...):
        """Get the last n lines from the history database.

        Most recent entry last.

        Completion will be reordered so that that the last ones are when
        possible from current session.

        Parameters
        ----------
        n : int
            The number of lines to get
        raw, output : bool
            See :meth:`get_range`
        include_latest : bool
            If False (default), n+1 lines are fetched, and the latest one
            is discarded. This is intended to be used where the function
            is called by a user command, which it should not return.

        Returns
        -------
        Tuples as :meth:`get_range`
        """
        ...
    
    def get_range(self, session=..., start=..., stop=..., raw=..., output=...):
        """Retrieve input by session.

        Parameters
        ----------
        session : int
            Session number to retrieve. The current session is 0, and negative
            numbers count back from current session, so -1 is previous session.
        start : int
            First line to retrieve.
        stop : int
            End of line range (excluded from output itself). If None, retrieve
            to the end of the session.
        raw : bool
            If True, return untranslated input
        output : bool
            If True, attempt to include output. This will be 'real' Python
            objects for the current session, or text reprs from previous
            sessions if db_log_output was enabled at the time. Where no output
            is found, None is used.

        Returns
        -------
        entries
            An iterator over the desired lines. Each line is a 3-tuple, either
            (session, line, input) if output is False, or
            (session, line, (input, output)) if output is True.
        """
        ...
    
    def store_inputs(self, line_num, source, source_raw=...): # -> None:
        """Store source and raw input in history and create input cache
        variables ``_i*``.

        Parameters
        ----------
        line_num : int
            The prompt number of this input.
        source : str
            Python input.
        source_raw : str, optional
            If given, this is the raw input without any IPython transformations
            applied to it.  If not given, ``source`` is used.
        """
        ...
    
    def store_output(self, line_num): # -> None:
        """If database output logging is enabled, this saves all the
        outputs from the indicated prompt number to the database. It's
        called by run_cell after code has been executed.

        Parameters
        ----------
        line_num : int
            The line number from which to save outputs
        """
        ...
    
    @only_when_enabled
    def writeout_cache(self, conn=...): # -> None:
        """Write any entries in the cache to the database."""
        ...
    


class HistorySavingThread(threading.Thread):
    """This thread takes care of writing history to the database, so that
    the UI isn't held up while that happens.

    It waits for the HistoryManager's save_flag to be set, then writes out
    the history cache. The main thread is responsible for setting the flag when
    the cache size reaches a defined threshold."""
    daemon = ...
    stop_now = ...
    enabled = ...
    def __init__(self, history_manager) -> None:
        ...
    
    @only_when_enabled
    def run(self): # -> None:
        ...
    
    def stop(self): # -> None:
        """This can be called from the main thread to safely stop this thread.

        Note that it does not attempt to write out remaining history before
        exiting. That should be done by calling the HistoryManager's
        end_session method."""
        ...
    


range_re = ...
def extract_hist_ranges(ranges_str):
    """Turn a string of history ranges into 3-tuples of (session, start, stop).

    Empty string results in a `[(0, 1, None)]`, i.e. "everything from current
    session".

    Examples
    --------
    >>> list(extract_hist_ranges("~8/5-~7/4 2"))
    [(-8, 5, None), (-7, 1, 5), (0, 2, 3)]
    """
    ...




from typing import (
    IO,
    Any,
)

from pandas._typing import FilePathOrBuffer

s3fs = ...

def get_file_and_filesystem(
    filepath_or_buffer: FilePathOrBuffer, mode: str | None = ...
) -> tuple[IO, Any]: ...
def get_filepath_or_buffer(
    filepath_or_buffer: FilePathOrBuffer,
    encoding: str | None = ...,
    compression: str | None = ...,
    mode: str | None = ...,
) -> tuple[IO, str | None, str | None, bool]: ...



from typing import Literal, Mapping
from threadpoolctl import _ThreadpoolLimiter
from ..externals._lobpcg import lobpcg as lobpcg
from .parallel import delayed
from .deprecation import deprecated
from numpy import percentile as percentile
from scipy.linalg import eigh as _eigh
from scipy.optimize.linesearch import (
    line_search_wolfe2 as line_search_wolfe2,
    line_search_wolfe1 as line_search_wolfe1,
)
from ..externals._packaging.version import parse as parse_version
from importlib import resources as resources
import sys

import sklearn
import numpy as np
import scipy
import scipy.stats
import threadpoolctl


np_version = ...
sp_version = ...


class loguniform(scipy.stats.reciprocal):
    ...


def threadpool_limits(
    limits: None | Mapping | str | int = None,
    user_api: None | Literal["blas", "openmp"] = None,
) -> _ThreadpoolLimiter:
    ...


threadpool_limits.__doc__ = ...


def threadpool_info() -> list[dict[str, int | str] | dict[str, str | None | int]]:
    ...


threadpool_info.__doc__ = ...


@deprecated(
    "The function `delayed` has been moved from `sklearn.utils.fixes` to "
    "`sklearn.utils.parallel`. This import path will be removed in 1.5."
)
def delayed(function):
    ...



from typing import Any, ClassVar, Literal, TypeVar
from scipy.special import comb as comb
from scipy import sparse
from itertools import (
    chain as chain,
    combinations as combinations,
    combinations_with_replacement as combinations_w_r,
)
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from numpy import ndarray
from numbers import Integral as Integral
from pandas.core.series import Series
from scipy.interpolate import BSpline as BSpline
from ..base import BaseEstimator, TransformerMixin
from scipy.sparse import spmatrix
from .._typing import ArrayLike, MatrixLike, Int
from ..utils import check_array as check_array
from ..utils.validation import (
    check_is_fitted as check_is_fitted,
    FLOAT_DTYPES as FLOAT_DTYPES,
)

SplineTransformer_Self = TypeVar("SplineTransformer_Self", bound="SplineTransformer")
PolynomialFeatures_Self = TypeVar("PolynomialFeatures_Self", bound="PolynomialFeatures")

import collections

import numpy as np


__all__ = [
    "PolynomialFeatures",
    "SplineTransformer",
]


class PolynomialFeatures(TransformerMixin, BaseEstimator):
    n_output_features_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        degree: int | tuple[int, int] = 2,
        *,
        interaction_only: bool = False,
        include_bias: bool = True,
        order: Literal["C", "F", "C"] = "C",
    ) -> None:
        ...

    @property
    def powers_(self) -> ndarray:
        ...

    def get_feature_names_out(self, input_features: None | ArrayLike = None) -> ndarray:
        ...

    def fit(
        self: PolynomialFeatures_Self, X: MatrixLike | ArrayLike, y: Any = None
    ) -> PolynomialFeatures_Self:
        ...

    def transform(self, X: MatrixLike | ArrayLike) -> ndarray | spmatrix:
        ...


# TODO:
# - sparse support (either scipy or own cython solution)?
class SplineTransformer(TransformerMixin, BaseEstimator):
    n_features_out_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    bsplines_: list = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_knots: Int = 5,
        degree: Int = 3,
        *,
        knots: Literal["uniform", "quantile", "uniform"] | MatrixLike = "uniform",
        extrapolation: Literal[
            "error", "constant", "linear", "continue", "periodic", "constant"
        ] = "constant",
        include_bias: bool = True,
        order: Literal["C", "F", "C"] = "C",
    ) -> None:
        ...

    def get_feature_names_out(self, input_features: None | ArrayLike = None) -> ndarray:
        ...

    def fit(
        self: SplineTransformer_Self,
        X: MatrixLike,
        y: Series | None | ndarray = None,
        sample_weight: None | ArrayLike = None,
    ) -> SplineTransformer_Self:
        ...

    def transform(self, X: MatrixLike) -> ndarray:
        ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.optimize._cobyla, version: $Revision: $
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
__version__: bytes
def minimize(calcfc, m, x, rhobeg, rhoend, dinfo, iprint=..., maxfun=..., calcfc_extra_args=...) -> typing.Any:
    "x,dinfo = minimize(calcfc,m,x,rhobeg,rhoend,dinfo,[iprint,maxfun,calcfc_extra_args])\n\nWrapper for ``minimize``.\n\nParameters\n----------\ncalcfc : call-back function\nm : input int\nx : input rank-1 array('d') with bounds (n)\nrhobeg : input float\nrhoend : input float\ndinfo : input rank-1 array('d') with bounds (4)\n\nOther Parameters\n----------------\ncalcfc_extra_args : input tuple, optional\n    Default: ()\niprint : input int, optional\n    Default: 1\nmaxfun : input int, optional\n    Default: 100\n\nReturns\n-------\nx : rank-1 array('d') with bounds (n)\ndinfo : rank-1 array('d') with bounds (4)\n\nNotes\n-----\nCall-back functions::\n\n  def calcfc(x,con): return f\n  Required arguments:\n    x : input rank-1 array('d') with bounds (n)\n    con : input rank-1 array('d') with bounds (m)\n  Return objects:\n    f : float\n"
    ...

def __getattr__(name) -> typing.Any:
    ...




# COMPLETE

from pathlib import Path
from typing import Any, Dict, List, Mapping, Union

_Style = Union[str, Path, Mapping[str, Any]]
_StyleOrList = Union[_Style, List[_Style]]

def context(style: _StyleOrList, after_reset: bool = ...) -> None: ...

def reload_library() -> None: ...

def use(style: _StyleOrList) -> None: ...

library: Dict[str, Any]
available: List[str]



from matplotlib.texmanager import TexManager
from typing import Literal, Sequence
from .path import Path
from .font_manager import FontProperties

class TextToPath:

    FONT_SCALE = ...
    DPI = ...
    def __init__(self) -> None: ...
    def get_text_width_height_descent(
        self, s: str, prop: FontProperties, ismath: bool | Literal["TeX"]
    ): ...
    def get_text_path(
        self, prop: FontProperties, s: str, ismath: bool | Literal["TeX"] = ...
    ): ...
    def get_glyphs_with_font(
        self, font: FontProperties, s: str, glyph_map=..., return_new_glyphs_only=...
    ): ...
    def get_glyphs_mathtext(
        self, prop: FontProperties, s: str, glyph_map=..., return_new_glyphs_only=...
    ): ...
    def get_texmanager(self) -> TexManager: ...
    def get_glyphs_tex(
        self, prop: FontProperties, s: str, glyph_map=..., return_new_glyphs_only=...
    ): ...

text_to_path = ...

class TextPath(Path):
    def __init__(
        self,
        xy: Sequence[float],
        s: str,
        size=...,
        prop: FontProperties = ...,
        _interpolation_steps: int = ...,
        usetex: bool = False,
    ) -> None: ...
    def set_size(self, size: float): ...
    def get_size(self) -> float: ...
    @property
    def vertices(self) -> Path: ...
    @property
    def codes(self): ...



from collections.abc import Iterable

import itertools
import numbers

from ..classes.graph import Graph
from ..classes import Graph
from ..exception import NetworkXError
from ..utils import nodes_or_number, pairwise

__all__ = [
    "balanced_tree",
    "barbell_graph",
    "binomial_tree",
    "complete_graph",
    "complete_multipartite_graph",
    "circular_ladder_graph",
    "circulant_graph",
    "cycle_graph",
    "dorogovtsev_goltsev_mendes_graph",
    "empty_graph",
    "full_rary_tree",
    "ladder_graph",
    "lollipop_graph",
    "null_graph",
    "path_graph",
    "star_graph",
    "trivial_graph",
    "turan_graph",
    "wheel_graph",
]

# -------------------------------------------------------------------
#   Some Classic Graphs
# -------------------------------------------------------------------

def full_rary_tree(r: int, n: int, create_using=None): ...
def balanced_tree(r: int, h: int, create_using=None): ...
def barbell_graph(m1, m2, create_using=None): ...
def binomial_tree(n: int, create_using=None): ...
@nodes_or_number(0)
def complete_graph(n, create_using=None): ...
def circular_ladder_graph(n, create_using=None): ...
def circulant_graph(n, offsets, create_using=None): ...
@nodes_or_number(0)
def cycle_graph(n, create_using=None): ...
def dorogovtsev_goltsev_mendes_graph(n, create_using=None): ...
@nodes_or_number(0)
def empty_graph(n=0, create_using=None, default=...): ...
def ladder_graph(n, create_using=None): ...
def lollipop_graph(m, n, create_using=None): ...
def null_graph(create_using=None): ...
@nodes_or_number(0)
def path_graph(n: int | Iterable, create_using=None): ...
@nodes_or_number(0)
def star_graph(n: int | Iterable, create_using=None): ...
def trivial_graph(create_using=None): ...
def turan_graph(n: int, r: int): ...
@nodes_or_number(0)
def wheel_graph(n: int | Iterable, create_using=None): ...
def complete_multipartite_graph(*subset_sizes): ...



from ._gaussian_mixture import GaussianMixture as GaussianMixture
from ._bayesian_mixture import BayesianGaussianMixture as BayesianGaussianMixture


__all__ = ["GaussianMixture", "BayesianGaussianMixture"]



from typing import Literal, Sequence
from ..utils import (
    check_array as check_array,
    check_matplotlib_support as check_matplotlib_support,
)
from ..ensemble import RandomForestRegressor as RandomForestRegressor
from scipy import sparse as sparse
from ..tree import DecisionTreeRegressor as DecisionTreeRegressor
from ..ensemble._gb import BaseGradientBoosting as BaseGradientBoosting
from ..exceptions import NotFittedError as NotFittedError
from ..base import BaseEstimator
from ..utils.extmath import cartesian as cartesian
from ..base import is_classifier as is_classifier, is_regressor as is_regressor
from ..utils._bunch import Bunch
from scipy.stats.mstats import mquantiles as mquantiles
from .._typing import MatrixLike, ArrayLike, Int
from collections.abc import Iterable as Iterable
from ..utils.validation import check_is_fitted as check_is_fitted
from ..ensemble._hist_gradient_boosting.gradient_boosting import (
    BaseHistGradientBoosting as BaseHistGradientBoosting,
)

import numpy as np


__all__ = [
    "partial_dependence",
]


def partial_dependence(
    estimator: BaseEstimator,
    X: MatrixLike,
    features: tuple[int]
    | Sequence[tuple[int, str]]
    | tuple[str, str]
    | tuple[int, int],
    *,
    categorical_features: None | MatrixLike | ArrayLike = None,
    feature_names: None | ArrayLike = None,
    response_method: Literal[
        "auto", "predict_proba", "decision_function", "auto"
    ] = "auto",
    percentiles: tuple[float, ...] = ...,
    grid_resolution: Int = 100,
    method: Literal["auto", "recursion", "brute", "auto"] = "auto",
    kind: Literal["average", "individual", "both", "average"] = "average",
) -> Bunch:
    ...



from typing import Sequence
from matplotlib.path import Path
from matplotlib.transforms import Transform
from matplotlib.backend_bases import MouseButton
from matplotlib.ticker import Formatter
from matplotlib.axes import Axes
import numpy as np

class GeoAxes(Axes):
    class ThetaFormatter(Formatter):
        def __init__(self, round_to=...) -> None: ...
        def __call__(self, x, pos=...): ...

    RESOLUTION = ...
    def clear(self)-> None: ...
    def get_xaxis_transform(self, which=...)-> Transform: ...
    def get_xaxis_text1_transform(self, pad)-> Transform: ...
    def get_xaxis_text2_transform(self, pad)-> Transform: ...
    def get_yaxis_transform(self, which=...)-> Transform: ...
    def get_yaxis_text1_transform(self, pad)-> Transform: ...
    def get_yaxis_text2_transform(self, pad)-> Transform: ...
    def set_yscale(self, *args, **kwargs)-> None: ...

    set_xscale = set_yscale
    def set_xlim(self, *args, **kwargs)-> None: ...
    set_ylim = set_xlim
    def format_coord(self, lon, lat)-> str: ...
    def set_longitude_grid(self, degrees: float)-> None: ...
    def set_latitude_grid(self, degrees: float)-> None: ...
    def set_longitude_grid_ends(self, degrees: float)-> None: ...
    def get_data_ratio(self)-> float: ...
    def can_zoom(self) -> bool: ...
    def can_pan(self) -> bool: ...
    def start_pan(self, x: float, y: float, button: MouseButton): ...
    def end_pan(self)-> bool: ...
    def drag_pan(self, button: MouseButton, key: str | None, x: float, y: float): ...

class _GeoTransform(Transform):
    input_dims = ...
    def __init__(self, resolution) -> None: ...
    def __str__(self) -> str: ...
    def transform_path_non_affine(self, path: Path)-> Path: ...

class AitoffAxes(GeoAxes):
    name: str = ...

    class AitoffTransform(_GeoTransform):
        def transform_non_affine(self, ll)-> np.ndarray: ...
        def inverted(self) -> _GeoTransform: ...

    class InvertedAitoffTransform(_GeoTransform):
        def transform_non_affine(self, xy: Sequence[float]): ...
        def inverted(self)-> _GeoTransform: ...

    def __init__(self, *args, **kwargs) -> None: ...

class HammerAxes(GeoAxes):
    name = ...

    class HammerTransform(_GeoTransform):
        def transform_non_affine(self, ll)-> np.ndarray: ...
        def inverted(self)-> _GeoTransform: ...

    class InvertedHammerTransform(_GeoTransform):
        def transform_non_affine(self, xy: Sequence[float])-> np.ndarray: ...
        def inverted(self)-> _GeoTransform: ...

    def __init__(self, *args, **kwargs) -> None: ...

class MollweideAxes(GeoAxes):
    name = ...

    class MollweideTransform(_GeoTransform):
        def transform_non_affine(self, ll)-> np.ndarray: ...
        def inverted(self)-> _GeoTransform: ...

    class InvertedMollweideTransform(_GeoTransform):
        def transform_non_affine(self, xy: Sequence[float])-> np.ndarray: ...
        def inverted(self)-> _GeoTransform: ...

    def __init__(self, *args, **kwargs) -> None: ...

class LambertAxes(GeoAxes):
    name = ...

    class LambertTransform(_GeoTransform):
        def __init__(self, center_longitude, center_latitude, resolution) -> None: ...
        def transform_non_affine(self, ll)-> np.ndarray: ...
        def inverted(self)-> _GeoTransform: ...

    class InvertedLambertTransform(_GeoTransform):
        def __init__(self, center_longitude, center_latitude, resolution) -> None: ...
        def transform_non_affine(self, xy: Sequence[float])-> np.ndarray: ...
        def inverted(self)-> _GeoTransform: ...

    def __init__(
        self, *args, center_longitude=..., center_latitude=..., **kwargs
    ) -> None: ...
    def clear(self)-> None: ...



"""
This type stub file was generated by pyright.
"""

from IPython.core.magic import Magics, line_magic, magics_class

"""Implementation of magic functions for IPython's own logging.
"""
@magics_class
class LoggingMagics(Magics):
    """Magics related to all logging machinery."""
    quiet = ...
    @line_magic
    def logstart(self, parameter_s=...): # -> None:
        """Start logging anywhere in a session.

        %logstart [-o|-r|-t|-q] [log_name [log_mode]]

        If no name is given, it defaults to a file named 'ipython_log.py' in your
        current directory, in 'rotate' mode (see below).

        '%logstart name' saves to file 'name' in 'backup' mode.  It saves your
        history up to that point and then continues logging.

        %logstart takes a second optional parameter: logging mode. This can be one
        of (note that the modes are given unquoted):

        append
            Keep logging at the end of any existing file.

        backup
            Rename any existing file to name~ and start name.

        global
            Append to  a single logfile in your home directory.

        over
            Overwrite any existing log.

        rotate
            Create rotating logs: name.1~, name.2~, etc.

        Options:

          -o
            log also IPython's output. In this mode, all commands which
            generate an Out[NN] prompt are recorded to the logfile, right after
            their corresponding input line. The output lines are always
            prepended with a '#[Out]# ' marker, so that the log remains valid
            Python code.

          Since this marker is always the same, filtering only the output from
          a log is very easy, using for example a simple awk call::

            awk -F'#\\[Out\\]# ' '{if($2) {print $2}}' ipython_log.py

          -r
            log 'raw' input.  Normally, IPython's logs contain the processed
            input, so that user lines are logged in their final form, converted
            into valid Python.  For example, %Exit is logged as
            _ip.magic("Exit").  If the -r flag is given, all input is logged
            exactly as typed, with no transformations applied.

          -t
            put timestamps before each input line logged (these are put in
            comments).

          -q 
            suppress output of logstate message when logging is invoked
        """
        ...
    
    @line_magic
    def logstop(self, parameter_s=...): # -> None:
        """Fully stop logging and close log file.

        In order to start logging again, a new %logstart call needs to be made,
        possibly (though not necessarily) with a new filename, mode and other
        options."""
        ...
    
    @line_magic
    def logoff(self, parameter_s=...): # -> None:
        """Temporarily stop logging.

        You must have previously started logging."""
        ...
    
    @line_magic
    def logon(self, parameter_s=...): # -> None:
        """Restart logging.

        This function is for restarting logging which you've temporarily
        stopped with %logoff. For starting logging for the first time, you
        must use the %logstart function, which allows you to specify an
        optional log filename."""
        ...
    
    @line_magic
    def logstate(self, parameter_s=...): # -> None:
        """Print the status of the logging system."""
        ...
    





from typing import Callable, ClassVar, Literal
from numpy.random import RandomState
from ..decomposition import PCA as PCA
from scipy.spatial.distance import pdist as pdist, squareform as squareform
from scipy import linalg as linalg
from numpy import ndarray
from ..utils._param_validation import (
    Interval as Interval,
    StrOptions as StrOptions,
    Hidden as Hidden,
)
from numbers import Integral as Integral, Real as Real
from ..neighbors import NearestNeighbors as NearestNeighbors
from scipy.sparse import csr_matrix as csr_matrix, issparse as issparse
from ..base import BaseEstimator
from ..metrics.pairwise import pairwise_distances as pairwise_distances
from time import time as time
from .._typing import MatrixLike, Int, Float, ArrayLike
from ..utils import check_random_state as check_random_state
from ..utils.validation import check_non_negative as check_non_negative

# Author: Alexander Fabisch  -- <afabisch@informatik.uni-bremen.de>
# Author: Christopher Moody <chrisemoody@gmail.com>
# Author: Nick Travers <nickt@squareup.com>
# License: BSD 3 clause (C) 2014

# This is the exact and Barnes-Hut t-SNE implementation. There are other
# modifications of the algorithm:
# * Fast Optimization for t-SNE:
#   https://cseweb.ucsd.edu/~lvdmaaten/workshops/nips2010/papers/vandermaaten.pdf

import warnings
import numpy as np

MACHINE_EPSILON = ...


def trustworthiness(
    X: MatrixLike,
    X_embedded: MatrixLike,
    *,
    n_neighbors: Int = 5,
    metric: str | Callable = "euclidean",
) -> float:
    ...


class TSNE(BaseEstimator):
    n_iter_: int = ...
    learning_rate_: float = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    kl_divergence_: float = ...
    embedding_: ArrayLike = ...

    _parameter_constraints: ClassVar[dict] = ...

    # Control the number of exploration iterations with early_exaggeration on
    _EXPLORATION_N_ITER: ClassVar[int] = ...

    # Control the number of iterations between progress checks
    _N_ITER_CHECK: ClassVar[int] = ...

    def __init__(
        self,
        n_components: Int = 2,
        *,
        perplexity: Float = 30.0,
        early_exaggeration: Float = 12.0,
        learning_rate: float | Literal["auto", "auto"] = "auto",
        n_iter: Int = 1000,
        n_iter_without_progress: Int = 300,
        min_grad_norm: Float = 1e-7,
        metric: str | Callable = "euclidean",
        metric_params: None | dict = None,
        init: MatrixLike | Literal["random", "pca", "pca"] = "pca",
        verbose: Int = 0,
        random_state: RandomState | None | Int = None,
        method: Literal["barnes_hut", "exact", "barnes_hut"] = "barnes_hut",
        angle: Float = 0.5,
        n_jobs: None | Int = None,
        square_distances: str | bool = "deprecated",
    ) -> None:
        ...

    def fit_transform(self, X: MatrixLike, y: None | ndarray = None) -> ndarray:
        ...

    def fit(self, X: MatrixLike, y=None) -> ndarray:
        ...



from typing import Any, TypeVar, overload

_SD = TypeVar("_SD", bound="SafeData")

class SafeData:
    def __html__(self: _SD) -> _SD: ...

class SafeText(str, SafeData):
    @overload
    def __add__(self, rhs: SafeText) -> SafeText: ...
    @overload
    def __add__(self, rhs: str) -> str: ...
    @overload
    def __iadd__(self, rhs: SafeText) -> SafeText: ...
    @overload
    def __iadd__(self, rhs: str) -> str: ...

SafeString = SafeText

def mark_safe(s: Any) -> SafeText: ...



"""Bump `pyproject.toml` with changes in `requirements.txt`."""

from pathlib import Path

import toml

PYPROJECT = Path("pyproject.toml")
SOURCE = ".tools" / PYPROJECT
DESTINATION = Path("pyproject.toml")
source = toml.loads(SOURCE.read_text(encoding="utf-8"))
DESTINATION.write_text(encoding="utf-8", data=toml.dumps(source))



from numpy.typing import ArrayLike
import numpy as np
from numpy.typing import NDArray

from .. import gloo
from .visual import Visual

_VERTEX_SHADER: str = ...

_FRAGMENT_SHADER: str = ...

class LinearRegionVisual(Visual):

    _shaders: dict = ...

    def __init__(
        self,
        pos: ArrayLike | tuple | NDArray | None = None,
        color: ArrayLike | tuple = [1.0, 1.0, 1.0, 1.0],
        vertical=True,
        **kwargs,
    ): ...
    def set_data(
        self,
        pos: ArrayLike | tuple | NDArray | None = None,
        color: ArrayLike | tuple | None = None,
    ): ...
    @property
    def color(self): ...
    @property
    def pos(self): ...
    def _compute_bounds(self, axis, view): ...
    @property
    def is_vertical(self): ...
    def _prepare_transforms(self, view=None): ...
    def _prepare_draw(self, view=None): ...




import pytest


# Passes the main worker's random seeds to workers
class XDistHooks:
    def pytest_configure_node(self, node) -> None: ...


def pytest_configure(config) -> None: ...
def pytest_report_header(config) -> list[str]|None: ...



from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np
from os import path as op

from ..util import load_data_file as load_data_file

# This is the package data dir, not the dir for config, etc.
DATA_DIR = ...

def load_iris() -> dict[str, np.ndarray]: ...
def load_crate() -> np.ndarray: ...
def pack_unit(value) -> np.ndarray: ...
def pack_ieee(value) -> np.ndarray: ...
def load_spatial_filters(packed: bool = True) -> tuple[np.ndarray, tuple[str, ...]]: ...



from email.errors import NonPrintableDefect
from re import Pattern
from .scale import AsinhScale, FuncScale, LogScale, SymmetricalLogScale
import numpy as np
from typing import Callable, Iterator, Literal
from matplotlib._typing import *

import functools
from collections.abc import Mapping

class _ColorMapping(dict):
    def __init__(self, mapping) -> None: ...
    def __setitem__(self, key, value)-> None: ...
    def __delitem__(self, key)-> None: ...

def get_named_colors_mapping()-> _ColorMapping: ...

class ColorSequenceRegistry(Mapping):
    def __init__(self) -> None: ...
    def __getitem__(self, item)-> list[Color]: ...
    def __iter__(self)-> Iterator[Color]: ...
    def __len__(self)-> int: ...
    def __str__(self) -> str: ...
    def register(self, name: str, color_list: list[Color])-> None: ...
    def unregister(self, name)-> None: ...

def is_color_like(c)-> bool: ...
def same_color(c1: Color, c2: Color)-> bool: ...
def to_rgba(c: np.ma.masked_array, alpha: float = ...) -> tuple: ...
def to_rgba_array(c: ArrayLike, alpha: float = ...) -> list: ...
def to_rgb(c: Color)-> tuple: ...
def to_hex(c: Color, keep_alpha: bool = ...) -> str: ...

cnames: dict[str, str] = ...
hexColorPattern: Pattern[str] = ...
rgb2hex = to_hex
hex2color = to_rgb

class ColorConverter:

    colors: _ColorMapping = ...
    cache: dict = ...
    @staticmethod
    def to_rgb(c: Color)-> tuple: ...
    @staticmethod
    def to_rgba(c: np.ma.masked_array, alpha: float = ...) -> tuple: ...
    @staticmethod
    def to_rgba_array(c: ArrayLike, alpha: float = ...) -> list: ...

colorConverter: ColorConverter = ...

class Colormap:
    def __init__(self, name: str, N:int=256) -> None: ...
    def __call__(
        self,
        X: float | int | np.ndarray | Scalar,
        alpha: float | ArrayLike | None = None,
        bytes: bool = False,
    ) -> tuple: ...
    def __copy__(self)-> Colormap: ...
    def __eq__(self, other: Colormap) -> bool: ...
    def get_bad(self)-> np.ndarray: ...
    def set_bad(self, color: Color='k', alpha: float|None=None): ...
    def get_under(self)-> np.ndarray: ...
    def set_under(self, color: Color='k', alpha: float|None=None): ...
    def get_over(self)-> np.ndarray: ...
    def set_over(self, color: Color='k', alpha: float|None=None): ...
    def set_extremes(self, *, bad=None, under=None, over=None)-> None: ...
    def with_extremes(self, *, bad=None, under: None=..., over: None=...)-> None: ...
    def is_gray(self)-> bool: ...
    def reversed(self, name: str|None = None)-> Colormap: ...
    def copy(self): ...

class LinearSegmentedColormap(Colormap):
    def __init__(self, name, segmentdata, N=..., gamma=...) -> None: ...
    def set_gamma(self, gamma)-> NonPrintableDefect: ...
    @staticmethod
    def from_list(name: str, colors, N: int = ..., gamma: float = ...)-> LinearSegmentedColormap: ...
    def reversed(self, name: str = ...) -> LinearSegmentedColormap: ...

class ListedColormap(Colormap):
    def __init__(self, colors: list, array, name: str = ..., N: int = ...) -> None: ...
    def reversed(self, name: str = ...) -> ListedColormap: ...

class Normalize:
    def __init__(self, vmin=..., vmax=..., clip=...) -> None: ...
    @property
    def vmin(self)-> float: ...
    @vmin.setter
    def vmin(self, value: float)-> None: ...
    @property
    def vmax(self)-> float: ...
    @vmax.setter
    def vmax(self, value: float)-> None: ...
    @property
    def clip(self)-> bool: ...
    @clip.setter
    def clip(self, value: bool)-> None: ...
    @staticmethod
    def process_value(value)-> tuple[np.ndarray, bool]: ...
    def __call__(self, value, clip: bool|None = None): ...
    def inverse(self, value): ...
    def autoscale(self, A)-> None: ...
    def autoscale_None(self, A)-> None: ...
    def scaled(self)-> bool: ...

class TwoSlopeNorm(Normalize):
    def __init__(self, vcenter, vmin=..., vmax=...) -> None: ...
    @property
    def vcenter(self): ...
    @vcenter.setter
    def vcenter(self, value)-> None: ...
    def autoscale_None(self, A)-> None: ...
    def __call__(self, value, clip=...): ...
    def inverse(self, value): ...

class CenteredNorm(Normalize):
    def __init__(self, vcenter=..., halfrange=..., clip=...) -> None: ...
    def autoscale(self, A)-> None: ...
    def autoscale_None(self, A)-> None: ...
    @property
    def vcenter(self): ...
    @vcenter.setter
    def vcenter(self, vcenter)-> None: ...
    @property
    def halfrange(self): ...
    @halfrange.setter
    def halfrange(self, halfrange)-> None: ...
    def __call__(self, value, clip: bool = ...): ...

def make_norm_from_scale(scale_cls, base_norm_cls=..., *, init=...): ...
@make_norm_from_scale(
    FuncScale, init=lambda functions, vmin=None, vmax=None, clip=False: None
)
class FuncNorm(Normalize): ...

@make_norm_from_scale(functools.partial(LogScale, nonpositive="mask"))
class LogNorm(Normalize): ...

@make_norm_from_scale(
    SymmetricalLogScale,
    init=lambda linthresh, linscale=1, vmin=None, vmax=None, clip=False, *, base=10: None,
)
class SymLogNorm(Normalize):
    @property
    def linthresh(self): ...
    @linthresh.setter
    def linthresh(self, value): ...

@make_norm_from_scale(
    AsinhScale, init=lambda linear_width=1, vmin=None, vmax=None, clip=False: None
)
class AsinhNorm(Normalize):
    @property
    def linear_width(self): ...
    @linear_width.setter
    def linear_width(self, value)-> None: ...

class PowerNorm(Normalize):
    def __init__(self, gamma, vmin=..., vmax=..., clip=...) -> None: ...
    def __call__(self, value, clip: bool = ...): ...
    def inverse(self, value): ...

class BoundaryNorm(Normalize):
    def __init__(self, boundaries, ncolors, clip=..., *, extend=...) -> None: ...
    def __call__(self, value, clip=...): ...
    def inverse(self, value): ...

class NoNorm(Normalize):
    def __call__(self, value, clip: bool = ...): ...
    def inverse(self, value): ...

def rgb_to_hsv(arr): ...
def hsv_to_rgb(hsv): ...

class LightSource:
    def __init__(
        self,
        azdeg=...,
        altdeg=...,
        hsv_min_val=...,
        hsv_max_val=...,
        hsv_min_sat=...,
        hsv_max_sat=...,
    ) -> None: ...
    @property
    def direction(self): ...
    def hillshade(
        self,
        elevation,
        vert_exag: float = ...,
        dx: float = ...,
        dy: float = ...,
        fraction: float = ...,
    ) -> np.ndarray: ...
    def shade_normals(self, normals, fraction: float = ...) -> np.ndarray: ...
    def shade(
        self,
        data,
        cmap: Colormap,
        norm=...,
        blend_mode: Literal["hsv", "overlay", "soft"] | Callable = ...,
        vmin: float | None = ...,
        vmax: float | None = ...,
        vert_exag: float = ...,
        dx: float = ...,
        dy: float = ...,
        fraction: float = ...,
        **kwargs
    ) -> np.ndarray: ...
    def shade_rgb(
        self,
        rgb: ArrayLike,
        elevation: ArrayLike,
        fraction: float = ...,
        blend_mode: Literal["hsv", "overlay", "soft"] | Callable = ...,
        vert_exag: float = ...,
        dx: float = ...,
        dy: float = ...,
        **kwargs
    ) -> np.ndarray: ...
    def blend_hsv(
        self,
        rgb: np.ndarray,
        intensity: np.ndarray,
        hsv_max_sat: float = ...,
        hsv_max_val: float = ...,
        hsv_min_val: float = ...,
        hsv_min_sat: float = ...,
    ) -> np.ndarray: ...
    def blend_soft_light(
        self, rgb: np.ndarray, intensity: np.ndarray
    ) -> np.ndarray: ...
    def blend_overlay(self, rgb: np.ndarray, intensity: np.ndarray) -> np.ndarray: ...

def from_levels_and_colors(
    levels, colors, extend: Literal["neither", "min", "max", "both"] = ...
): ...



from ...classes.graph import Graph

__all__ = [
    "union",
    "compose",
    "disjoint_union",
    "intersection",
    "difference",
    "symmetric_difference",
    "full_join",
]

def union(G: Graph, H, rename: tuple = ..., name: str | None = None): ...
def disjoint_union(G: Graph, H): ...
def intersection(G: Graph, H): ...
def difference(G: Graph, H): ...
def symmetric_difference(G: Graph, H): ...
def compose(G: Graph, H): ...
def full_join(G: Graph, H, rename: tuple = ...): ...
